# Title: TestToCompareUHOutputToStandards
# 	Description: This program is designed to test UnifiedHydro modifications and ensure that deliberate changes have not introduced bugs. 
#	The program tests if UnifiedHydro.exe will generate output equal to expected output for known input of test cases. 

# Software Requirements: Python 3.6 64-bit
# Input:
#               - UnifiedHydro.exe: This is the executable you are testing AND must be placed in same directory as this script. 
#					Every time you want to test, copy from Debug or x64\Debug your latest build .exe to this directory.
#cte: - this seems a weak link in process of testing, and we may want Python script to go to Debug folder, adding that path to ListofTests input
#
#               - ListOfTests.txt: This file should be placed in the same directory as the Python script. 
#                   Executable line: Executable,Path where Executable is keyword for script, and Path is for executable UnifiedHydro.exe file
#					Test header line: TestName,InputFolder,ExpectedOutputFolder,ConfigFile
#					Next lines: values for each of the 4 header variables, separated by commas, variables defined below
#					    TestName = arbitrary name and allows you to find the results for this run in the output folder. 
#					    InputFolder = relative path to model input folder.
#					    ExpectedOutputFolder = relative path to saved expected output folder. 
#					    ConfigFile = relative path and name of config file, typically UnifiedHydroConfig.xml within inputs folder.
#						    The <OutputDirectory> in the config file is ignored by the Python script, using instead a newly created directory  

#                   Below are commented lines with an example header and inputs this file.
#Executable,C:\iTree\UnifiedHydro\UnifiedHydro\x64\Debug\UnifiedHydro.exe
#TestName,InputFolder,ExpectedOutputFolder,ConfigFile
#Hydro_powIR_defaultParams,TestCases\StatHydro\powIR_defaultParams\input,TestCases\StatHydro\powIR_defaultParams\expectedoutput,TestCases\StatHydro\powIR_defaultParams\input\UnifiedHydroConfig.xml
#
#					Note: when using with i-Tree Hydro GUI, 
#						1) TestName requires ! at start, as in "!TestName", to indicate use of old parameter names and config file format
#						2) ConfigFile = name of confg file, as in "hydro_config_file.xml", not the path  
#						Troubleshooting: if changes to parameter names break code, see code at void Inputs::readOldConfigParamFiles() 
#
# Output:       - Testing_%Y_%m_%d_%H_%M_%S folder created in same directory as Python script, with unique values for %Y_%m_%d_%H_%M_%S of test
#					results.txt file: defines test name, path to output and expectedoutput, and test results (Passed or Failed)
#					TestName folder: contains folders for input (with config file and <OutputDirectory>), output, and expectedoutput
# 					A Passed test has new output values that are indentical to the expected output values
#					A Failed test has new output values that are different to the expected output values, or is missing expected output files. 
#                	
# How to Run: 
#				Open Windows Command Prompt, go to directory with Python script, e.g.,C:\>cd C:\iTree\UnifiedHydro\TestingFilesAndScript. 
#				C:\iTree\UnifiedHydro\TestingFilesAndScript>py TestToCompareUHOutputToStandards.py
# Notes: 		
# Original Authors: Shannon Conley, 2018
#
# 2018/11/16:   S. Conley, Added code to test UnifiedHydro with old param.dat and hydro_config_file.xml
# 2019/10/10:   T. Endreny, Modified directions above, and added comments to previously uncommented code below
#                   Modified ListOfTests.txt to include path to executable file in 1st line
#                   Modified code below to read in path to executable file, UnifiedHydro.exe
# ===============================================================================

#import Python libraries
import subprocess
import os
import pandas as pd
import time
from random import uniform
import numpy as np
import xml.etree.ElementTree as ET
import csv 
import shutil
import glob
from filecmp import dircmp

#function to change the value of an element of a file path
def modify_params(path, element, value):
	#define path to file
    tree = ET.parse(path)
	#find element in file path
    a = tree.find(element)
	#define text of element = to string of input value 
    a.text = str(value)
	#write new text value of element to file path
    tree.write(path)

#function to change value of attribute of an element of a file path used for non-standard cases
def modify_attribute(path, element, attrName, value):
	#define content with {}
    content = {}
    with open(path) as f:
        content = f.readlines()
        count = 0;
        for str in content:
            if "<"+element in str:
                str = "<"+element+" "+attrName+"=\""+value+"\">"
            content[count] = str
            count = count + 1
        f.close()
        with open(path, "w") as f:
            str = "<"+element+" "+attrName+"="+value+">"
            for o in content:
                f.write(o)
        f.close()

#define test_file as input ListOfTests.txt
test_file = "ListOfTests.txt"

#open test_file as f
with open(test_file) as f:
	#define data for each line in f with tuple command, reading objects separated by commas
    data=[tuple(line) for line in csv.reader(f)]
	
#define timeN folder as Testing_%Y_%m_%d_%H_%M_%S, with variables for time and date components given by strftime() method
timeN = time.strftime("Testing_%Y_%m_%d_%H_%M_%S")

#define outputFile as results.txt within timeN folder
outputFile = timeN+"\\results.txt"

#define dictTests as strings to keep track of results
dictTests = {"Overall Test Results": "Passed"}

#for each testCase in data
for testCase in data:
     if testCase[0] == "Executable":
            #name UnifiedHydro executable
            hydro = testCase[1]
            continue
	#if 1st variable of data is "TestName" then treat as header file and continue to top of loop, and read in next line
     if testCase[0] == "TestName":
            continue
	#define foldName within timeN folder as 1st variable in data testCase, the unique test case name 
     foldName = time.strftime(timeN+"\\"+testCase[0])
	#define newoutpath within foldName folder with name for output
     newoutpath = os.path.join(os.path.dirname(__file__),foldName+'\\output')
	#define expoutpath within foldName folder with name for expectedoutput
     expoutpath = os.path.join(os.path.dirname(__file__),foldName+'\\expectedoutput')
	#define newinpath within foldName folder with name for input
     newinpath = os.path.join(os.path.dirname(__file__),foldName+'\\input')
	#make newoutpath if it does not exit
     if not os.path.exists(newoutpath):
        os.makedirs(newoutpath)
	#make newinpath if it does not exit
     if not os.path.exists(newinpath):
        os.makedirs(newinpath)
	#make expoutpath if it does not exit
     if not os.path.exists(expoutpath):
        os.makedirs(expoutpath)
		
	#define tc as input path, 2nd variable in data testCase, as upper case string
     tc = testCase[1].upper()
	 #for each file in the input path +/*.*, finding all file paths with glob command
     for file in glob.glob(tc+'/*.*'):
		#copy file to newinpath directory
        shutil.copy(file, newinpath)

	#define tc as expectedoutput path, 3nd variable in data testCase, as upper case string
     tc = testCase[2].upper()
	 #for each file in the expectedoutput path +/*.*, finding all file paths with glob command
     for file in glob.glob(tc+'/*.*'):
		#copy file to newinpath directory
        shutil.copy(file, expoutpath)
	#if 1st character of 1st variable is not "!" then proceed for standard use of script
     if testCase[0][0] != "!":
		#define name as config file within newinpath
        name = newinpath+'\\UnifiedHydroConfig.xml'
		#copy 4th variable in data testCase .xml to name file
        shutil.copy(testCase[3], name)

		#call function to modify newinpath .xml file variable, <OutputDirectory>, to contain path to newoutpath
        modify_params(newinpath+"\\UnifiedHydroConfig.xml","SimulationStringParams/OutputDirectory", newoutpath+"\\")

		#call UnifiedHydro.exe newinpath; this runs model with .xml and inputs in newinpath
        subprocess.call([hydro,newinpath])

	#if 1st character is "!" then proceed for non-standard use of script, using older parameters and config file
     else:
		#define newPathConfig as 4th variable in data testCase, name of config file
        newPathConfig = newinpath+"\\"+testCase[3]

		#call function to modify attributes for inputs in config file
        modify_attribute(newPathConfig, "Inputs","Input_Folder", newinpath+"//")
		#call function to modify attributes for outputs in config file
        modify_attribute(newPathConfig, "Outputs","Output_Folder", newoutpath+"//")

		#call UnifiedHydro.exe newinpath; this runs model with .xml and inputs in newinpath
        subprocess.call([hydro,newPathConfig])

	#define fc1 as output from dircmp direct compariosn between newoutpath and expoutpath files
     fc1 = dircmp(newoutpath, expoutpath)
	
	#open outputFile results.txt as f
     with open(outputFile,'a') as f:
        #define as default, dictTests for each testCase 1st variable as Passed
         dictTests[testCase[0]] =  "Passed"
        #write testCase name
         f.write("Test Name:"+testCase[0]+"\n")
        #write testCase path of output string and actual path
         f.write("Path of test generated output:")
         f.write(newoutpath)
         f.write("\n")
        #write testCase path of expectedoutput string and actual path
         f.write("Path of expected/standard output:")
         f.write(expoutpath)
         f.write("\n")
        #for each instance of the right_list, expectedoutput, of fc1 dircmp object
         for r in fc1.right_list:
            #if the instance of the right_list is not in the left_list, change default from Passed to Failed
             if r not in fc1.left_list:
                #write test case failed due to missing file outputs, and list missing file
                f.write("\t Test case failed.\n The following file is in the expected outputs, but missing from test dir.\n\t")
                #write right_list file
                f.write(r)
                f.write("\n")
                #write test results failed as result
                dictTests["Overall Test Results"]="Failed"
                dictTests[testCase[0]] = "Failed"
		#if diff_files function has found different files in output and expected output, change default from Passed to Failed
         if len(fc1.diff_files) != 0:
			#write why test case failed
             f.write("Test case failed! These file(s) are different:")
			#write overall test results as failed
             dictTests["Overall Test Results"]= "Failed"
			#write specific test case result as failed
             dictTests[testCase[0]] =  "Failed"
			#for diff in list of different files
             for diff in fc1.diff_files:
                #write diff file to f
                f.write(diff+ " ")
                f.write("\n")
         f.write("==============================================================================\n")
         #close outputFile
         f.close()
		 
#print to screen out dictTests
print(dictTests)

#open outputFile results.txt as f
with open(outputFile,'a') as f:
    #for each testCase result in dictTests
  for testRes in dictTests:
         #write result of testCase
         f.write(testRes + ":\t"+ str(dictTests[testRes]))
         
         f.write("\n")
f.close()

