#include "FlowRouteCalc.h"
#include <math.h>
#include <fstream>
#include <vector>
#include <iostream>
#include <numeric>
#include <string>

// Runoff Routing Function
// History: Developed by Yang and Endreny (2013) and placed in iTree V5, as a 2-parameter diffusion hydrograph; replacing the time-area hydrograph developed by Wang et al. (2008)
// References: 1. Wang, Jun, Theodore A. Endreny, and David J. Nowak. "Mechanistic Simulation of Tree Effects in an Impervious Water Balance Model1." JAWRA Journal of the American Water Resources Association Vol(44)1: 75-85 (2008)
//             2. Yang, Yang, Theodore A. Endreny. "Watershed hydrograph model based on surface flow diffusion", Water Resources Research Vol 49:507-516 (2013): 
//The 2 time-based parameters used by the model, alpha and beta, represent watershed scale, flow diffusivity, and flow celerity
//This 2-parameter diffuison model has excellent representation of natural hydrograph characteristics, including the inflection points in the rising and falling limbs, a fast rising limb and a gradual falling limb, and time to peak.
/*
	A visual demonstration of the influence of alpha and beta on the normalized hydrograph is shown in Figure 4 of Yang and Endreny (2013). 
	Setting beta = 1, and varying alpha from 0.1 and 10, alpha and beta together determine the rising limb, tmax, and falling limb when (Figure 4a); 
	The smaller alpha, the faster the rising and falling limb and the smaller tmax. 
	Setting beta = 1 and alpha >10, beta determines the hydrograph shape and tmax = 2(beta/3). 
	Setting alpha = 1 and varying beta from 0.1 to 100, the normalized hydrographs series indicate that beta always contributes to tmax (Figure 4b).
	The larger beta, the longer the hydrograph duration.
*/
//  The two time constants alpha and beta in the diffusion model have direct physical representation of watershed and weather controls on runoff and hydrograph features
//  By assigning the surface flow and the subsurface flow with the same time constants alpha and beta, the two-parameter model can be applied to simulate discharge with mixed surface flow and subsurface flow.
//  When surface flow and subsurface flow have different time constants alpha and beta, a parallel model that consists of two two-parameter models can be constructed to simulate the hydrograph
// Routing can separate or combine DCIA and PA components. We currently separate, but origionally combined. Baseflow is routed separately. 
	// Notes on alpha and beta, from Yang and Endreny (2013):
	// When alpha approaches infinity (the kinematic flow celerity is 0), the exponential term goes to 1,
	// and the 2-parameter surface flow diffusion hydrograph model (developed by Yang and Endreny) has the same form as
	// the 1-parameter subsurface flow diffusion hydrograph model of Criss and Winston [2008b].
	// The 2-parameter surface flow diffusion hydrograph model is used for PerviousFlow (PAQ) and Impervious Flow (IAQ).
	// The 1-parameter subsurface flow diffusion hydrograph model is used for Baseflow (BaseQ).
	// The time input, j, is used in calculating the discharge from the precip (pulse input) at time j in the future

void FlowRouteCalc::calculateStatistical(Inputs *input, int timeStep)
{
	int IT = timeStep;
	int IIN = 0;	// The for-loop controller - equals to the timestep + the instance of the for-loop
	auto const stride = 3; 
	int totalTs = input->SimulationNumericalParams["TotalTimeSteps"];

	if (timeStep == 0)
	{

		input->QQi.insert(input->QQi.end(), totalTs, 0.0);
		input->BQi.insert(input->BQi.end(), totalTs, 0.0);
		input->PAQi.insert(input->PAQi.end(), totalTs, 0.0);
		input->DCIAQi.insert(input->DCIAQi.end(), totalTs, 0.0);
		input->QQi[0] = input->RefParams["Q0_mph"];
	}

	//Define pervious, impervious, and baseflow runoff terms using unrouted runoff depths from FLowSumCalc.cpp
	double PAQ = input->PAQi_NR[timeStep];
	double DCIAQ = input->DCIAQi_NR[timeStep];
	double BQ = input->BQi_NR[timeStep];

	//cte 20200918 developing an alternative routing option to the Yang and Endreny (2013) 2-parameter diffusion hydrograph
	//	when flag = 1 this alternative hydrograph, likely Manning-based, will operate after the 2-parameter code
	double routing_flag = 0;
	if (routing_flag == 0)
	{

		double pa_alpha = input->RefParams["PAQ_RT_A_h"];		// Time parameter of the two parameter surface flow diffusion hydrograph model
		double pa_beta = input->RefParams["PAQ_RT_B_h"];		// Time parameter of the two parameter surface flow diffusion hydrograph model
		double dcia_alpha = input->RefParams["DCIAQ_RT_A_h"];
		double dcia_beta = input->RefParams["DCIAQ_RT_B_h"];
		double bq_b = input->RefParams["SSQ_RT_h"];

		double tmax_pa = (-0.75 * pa_alpha) + (0.75 * pa_alpha) * sqrt(1 + (16.0 / 9.0) * (pa_beta / pa_alpha));
		double tmax_dcia = (-0.75 * dcia_alpha) + (0.75 * dcia_alpha) * sqrt(1 + (16.0 / 9.0) * (dcia_beta / dcia_alpha));
		double tmax_bq = (2.0 / 3.0) * bq_b;

		// The routine implements a for loop to delay runoff from the current timestep, (int j = 0; j < tupper; j++),
		//		in effect distributing (delaying) some of the current runoff to future timesteps 
		//		tupper is set by 10*(b * 7.435 + 4.73).  When b is 1.3, tupper becomes 143, instead of 143.9, due to being static cast as an integer
		//		tupper is the time it takes for less than X% of Qmax (due to the precip pulse) to be left
		//		The tupper coefficients were derived by Yang Yang, examining the relationship of beta and the time to X% of Qmax

		// different max distirbution times - separated for surface flows and subsurface flows
		int tupper_pa = static_cast<int>(10 * ((tmax_pa * 7.435) + 4.73));
		int tupper_dcia = static_cast<int>(10 * ((tmax_dcia * 7.435) + 4.73));
		int tupper_bq = static_cast<int>(10 * ((tmax_bq * 7.435) + 4.73));

		// for water balance tracking - Tom T
		double sumDelay_BQ_TS = 0.0;
		double sumDelay_PAQ_TS = 0.0;
		double sumDelay_DCIAQ_TS = 0.0;
		double paq_it = 0.1;
		double bq_it = 0.1;
		double dciaq_it = 0.1;

		//PAQ (pervious area runoff) computation with Yang and Endreny (2013)
		if (PAQ > 0.0)
		{
			// for each spot in the vector
			for (int j = 0; j < tupper_pa && IT + j < totalTs; ++j)
			{
				IIN = IT + j;    // IIN  = timestep + j

				double first = PAQ / sqrt(3.14) * sqrt(pa_beta) * pow(paq_it, -1.5) * exp(-pa_beta / paq_it) * exp(-paq_it / pa_alpha) * exp(2 * sqrt(pa_beta / pa_alpha));
				if (j == 0)
					paq_it = 0.0;
				paq_it += 1;
				double second = PAQ / sqrt(3.14) * sqrt(pa_beta) * pow(paq_it, -1.5) * exp(-pa_beta / paq_it) * exp(-paq_it / pa_alpha) * exp(2 * sqrt(pa_beta / pa_alpha));
				paq_it += 1;
				double third = PAQ / sqrt(3.14) * sqrt(pa_beta) * pow(paq_it, -1.5) * exp(-pa_beta / paq_it) * exp(-paq_it / pa_alpha) * exp(2 * sqrt(pa_beta / pa_alpha));
				paq_it += 1;

				double sumDelay_PAQ_TS = first + second + third; //std::accumulate(paq_it, paq_it + stride, 0.0);
				// Pervious Flow (at timestep IIN) is modified to be equal to the Pervious flow (at IIN) + delayPAQ
				input->PAQi[IIN] += sumDelay_PAQ_TS;
				//Total Flow (at timestep IIN) is updated with the baseflow amount at timestep IIN
				input->QQi[IIN] += sumDelay_PAQ_TS;

			}
		}

		//DCIA (directly connected impervious area runoff) computation with Yang and Endreny (2013)
		if (DCIAQ > 0.0)
		{
			// for each spot in the vector
			for (int j = 0; j < tupper_dcia && IT + j < totalTs; ++j)//
			{
				IIN = IT + j;    // IIN  = timestep + j

				//auto sum = std::accumulate(b, b + stride, 0);
				double first = DCIAQ / sqrt(3.14) * sqrt(dcia_beta) * pow(dciaq_it, -1.5) * exp(-dcia_beta / dciaq_it) * exp(-dciaq_it / dcia_alpha) * exp(2 * sqrt(dcia_beta / dcia_alpha));
				if (j == 0)
					dciaq_it = 0;
				dciaq_it += 1;
				double second = DCIAQ / sqrt(3.14) * sqrt(dcia_beta) * pow(dciaq_it, -1.5) * exp(-dcia_beta / dciaq_it) * exp(-dciaq_it / dcia_alpha) * exp(2 * sqrt(dcia_beta / dcia_alpha));
				dciaq_it += 1;
				double third = DCIAQ / sqrt(3.14) * sqrt(dcia_beta) * pow(dciaq_it, -1.5) * exp(-dcia_beta / dciaq_it) * exp(-dciaq_it / dcia_alpha) * exp(2 * sqrt(dcia_beta / dcia_alpha));
				dciaq_it += 1;

				//first second and third defined as RHS of sumDelay_DCIAQ_TS = *dciaq_it + *(dciaq_it + 1) + *(dciaq_it + 2); 
				double sumDelay_DCIAQ_TS = first + second + third;

				// Impervious Flow (at timestep IIN) is modified to be equal to the Impervious flow (at IIN) + delayPAQ
				input->DCIAQi[IIN] += sumDelay_DCIAQ_TS;
				//Total Flow (at timestep IIN) is updated with the baseflow amount at timestep IIN
				input->QQi[IIN] += sumDelay_DCIAQ_TS;
			}
		}

		//BQ (baseflow runoff) computation with Yang and Endreny (2013)
		if (BQ > 0.0)
		{
			for (int j = 0; j < tupper_bq && IT + j < totalTs; ++j) {

				IIN = IT + j;    // IIN  = timestep + j==

				//if (IIN >= totalTs)  // Parameter24 is the total timesteps
					//break;

				double first = BQ / sqrt(3.14) * sqrt(bq_b) * pow(bq_it, -1.5) * exp(-bq_b / bq_it);
				if (j == 0)
					bq_it = 0;
				bq_it += 1;
				double second = BQ / sqrt(3.14) * sqrt(bq_b) * pow(bq_it, -1.5) * exp(-bq_b / bq_it);
				bq_it += 1;
				double third = BQ / sqrt(3.14) * sqrt(bq_b) * pow(bq_it, -1.5) * exp(-bq_b / bq_it);
				bq_it += 1;

				double sumDelay_BQ_TS = first + second + third;

				// Base Flow (at timestep IIN) is modified to be equal to the base flow (at IIN) + delayPAQ
				input->BQi[IIN] += sumDelay_BQ_TS;
				//Total Flow (at timestep IIN) is updated with the baseflow amount at timestep IIN
				input->QQi[IIN] += sumDelay_BQ_TS;
			}
		}
	}
	else
	{
		//cte 20200918 developing an alternative routing option to the Yang and Endreny (2013) 2-parameter diffusion hydrograph
		// cte 20200918 testing alternative writing out of PAQ, DCIAQ, BQ, and QQ
		IIN = timeStep;
		//Manning discharge terms
		//PAQ is depth (m), W is watershed width (m), n is Manning coefficient, S is surface slope (m/m)
		double W_paq = 484;
		double n_paq = 0.24;
		double S_paq = 0.02;
		double W_dcia = 484;
		double n_dcia = 0.015;
		double S_dcia = 0.02;

		// Manning discharge from Eq 3-3 from SWMM Reference Manual Volume I  Hydrology (Revised) 2016 by L. Rossman & W. Huber | EPA/600/R-15/162A | January 2016 | www2.epa.gov/water-research
		// Q = 1/n * Width * Depth_runoff^(5/3) * Slope^(1/2), where [Width * Depth_runoff] = Area_xs, and [Width * Depth_runoff] * Depth_runoff^(2/3) = Width * Depth_runoff^(5/3) 
		// Need to implement algorithm for creating hydrograph, constrained by available volume per time step, and updating depth
		input->PAQi[IIN] += (1/n_paq * W_paq * pow(PAQ,(5/3)) * pow(S_paq, (1 / 2)));
		input->DCIAQi[IIN] += (1 / n_dcia * W_dcia * pow(DCIAQ, (5 / 3)) * pow(S_dcia, (1 / 2)));
		input->BQi[IIN] += BQ;
		input->QQi[IIN] += input->BQi[IIN] + input->DCIAQi[IIN] + input->PAQi[IIN];
	}

	// Adding to the total runoffs
	//repo->runoff.setRunTotalQQ(repo->runoff.getRunTotalQQ() + repo->runoff.getinput->QQi(timeStep));
	input->RepoDict["runTotalFlow_m"] += input->QQi[timeStep];
	//repo->runoff.setRunTotalBQ(repo->runoff.getRunTotalBQ() + repo->runoff.getinput->BQi(timeStep));
	input->RepoDict["runTotalSubsurfaceRunoff_m"] += input->BQi[timeStep];
	//repo->runoff.setRunTotalPAQ(repo->runoff.getRunTotalPAQ() + repo->runoff.getinput->PAQi(timeStep));
	input->RepoDict["runTotalPerviousAreaRunoff_m"] += input->PAQi[timeStep];
	//repo->runoff.setRunTotalDCIAQ(repo->runoff.getRunTotalDCIAQ() + repo->runoff.getinput->DCAQi(timeStep));
	input->RepoDict["runTotalDCImpAreaRunoff_m"] += input->DCIAQi[timeStep];

	///////////////////////////////////////////////////////////////////////////////////////////////


	/*
	////////////////////////////////////////////////////////////////////////////////////////////////
	// This is the orinal routing code
	// Notes on alpha and beta, from Yang and Endreny (2013):
	// When alpha approaches infinity (the kinematic flow celerity is 0), the exponential term goes to 1,
	// and the 2-parameter surface flow diffusion hydrograph model (developed by Yang and Endreny) has the same form as
	// the 1-parameter subsurface flow diffusion hydrograph model of Criss and Winston [2008b].
	// The 2-parameter surface flow diffusion hydrograph model is used for PerviousFlow (PAQ) and Impervious Flow (IAQ).
	// The 1-parameter subsurface flow diffusion hydrograph model is used for Baseflow (BaseQ).
	// The time input, j, is used in calculating the discharge from the precip (pulse input) at time j in the future

	//This for loop delays runoff from the current timestep, in effect distributing (delaying) some of the current runoff to future timesteps
	for (int j=0;j<tupper;j++)
	// tupper is set by 10*(b * 7.435 + 4.73).  When b is 1.3, tupper becomes 143, instead of 143.9, due to being static cast as an integer
	// tupper is the time it takes for less than X% of Qmax (due to the precip pulse) to be left
	// The coefficients in the tupper equation were derived by Yang Yang, examining the relationship of beta and the time to X% of Qmax
	{

	if(j==0)
	{
	// Pervious flow uses the two parameter surface flow diffusion hydrograph model
	delayPAQ= PAQ/sqrt(3.14)*sqrt(beta)*pow(0.1,-1.5)*exp(-beta/0.1)*exp(-0.1/alpha)*exp(2*sqrt(beta/alpha));
	// Impervious flow (impervious flow) uses the two parameter surface flow diffusion hydrograph model
	delayIAQ= IAQ/sqrt(3.14)*sqrt(beta)*pow(0.1,-1.5)*exp(-beta/0.1)*exp(-0.1/alpha)*exp(2*sqrt(beta/alpha));
	// Base flow uses the one parameter subsurface flow diffusion hydrograph model
	delayBaseQ=BQ/sqrt(3.14)*sqrt(b)*pow(0.1,-1.5)*exp(-b/0.1);
		}
	else
	{
	// Pervious flow uses the two parameter surface flow diffusion hydrograph model
	delayPAQ= PAQ/sqrt(3.14)*sqrt(beta)*pow(j,-1.5)*exp(-beta/j)*exp(-j/alpha)*exp(2*sqrt(beta/alpha));
	// Impervious flow (impervious flow) uses the two parameter surface flow diffusion hydrograph model
	delayIAQ= IAQ/sqrt(3.14)*sqrt(beta)*pow(j,-1.5)*exp(-beta/j)*exp(-j/alpha)*exp(2*sqrt(beta/alpha));
	// Base flow uses the one parameter subsurface flow diffusion hydrograph model
	delayBaseQ=BQ/sqrt(3.14)*sqrt(b)*pow(j,-1.5)*exp(-b/j);
	}

	IIN = IT + j;    // IIN  = timestep + j
	if( IIN >= totalTs)  // Parameter24 is the total timesteps
	break;
	// If the current timestep + j (the hours into the future we are distributing flow) > the total timesteps, break
	// Meaning, there is no point to distributing flow to the hours beyond the model simulation
	// Pervious Flow (at timestep IIN) is modified to be equal to the Pervious flow (at IIN) + delayPAQ
	repo->runoff.modifyinput->PAQi(repo->runoff.getinput->PAQi(IIN) + delayPAQ, IIN);
	// Impervious Flow (at timestep IIN) is modified to be equal to the Impervious flow (at IIN) + delayPAQ
	repo->runoff.modifyIAQi(repo->runoff.getIAQi(IIN) + delayIAQ, IIN);
	// Base Flow (at timestep IIN) is modified to be equal to the base flow (at IIN) + delayPAQ
	repo->runoff.modifyinput->BQi(repo->runoff.getinput->BQi(IIN) + delayBaseQ, IIN);
	
	//Total Flow (at timestep IIN) is modified to be equal to the all the flow components (at IIN)
	//Old code:
	//repo->runoff.modifyinput->QQi(repo->runoff.getinput->QQi(IIN) + (delayPAQ + delayIAQ + delayBaseQ), IIN);
	//repo->runoff.setTotalOLQ(repo->runoff.getTotalOLQ() + delayPAQ);
	//repo->runoff.setRunTotalBQ(repo->runoff.getRunTotalBQ() + delayBaseQ);
	//repo->runoff.setTotalUrImpQ(repo->runoff.getTotalUrImpQ() + delayIAQ);
	}
	repo->runoff.setTotalQQ(repo->runoff.getRunTotalQQ() + repo->runoff.getinput->QQi(timeStep));
	*/


}