#include "GI_Outlets.h"

//These GiLayerTypes are used in a generic function for the water balance
enum GILayerTypes {
	SURF,                    // surface layer
	SOIL,                    // soil layer
	STOR,                    // storage layer
	PAVE,                    // pavement layer
	DRAIN					 // underdrain system
};

//-----------------------------------------------------------------------------
//  Globally shared variables 
//-----------------------------------------------------------------------------

double     Vevap;         // evaporation
double     Vpevap;        // pervious area evaporation
double     Vinfil;        // non-GI infiltration
double     VGiInfil;     // infiltration from GI units

static double     SurfaceInflow;  // precip. + runon to GI unit (m/dt)
static double     SurfaceInfil;   // infil. rate from surface layer (m/dt)
static double     SurfaceEvap;    // evap. rate from surface layer (m/dt)
static double     SurfaceOutflow; // outflow from surface layer (m/dt)
static double     SurfaceVolume;  // volume in surface storage (m)

static double     EvapRate;       // evaporation rate (m/dt)
static double     SoilEvap;       // evap. from soil layer (m/dt)
static double     PaveEvap;       // evap. from pavement layer (m/dt)
static double     StorageEvap;    // evap.rate from storage layer (m/dt)
static double     SoilVolume;     // volume in soil/pavement storage (m)
static double     StorageVolume;  // volume in storage layer (m)
static double	  PavePerc;		  // percolation from pavement layer (m/dt)
static double     PaveVolume;     // volume stored in pavement layer  (m)

static double     SoilPerc;       // percolation from soil layer (m/dt)
static double     StorageExfil;   // exfil. rate from storage layer (m/dt)
static double     StorageDrain;   // underdrain flow rate layer (m/dt)
static double     StorageInflow;  // inflow rate to storage layer (m/dt)

static double Xold[MAX_LAYERS];  // previous moisture level in GI layers
int surfaceCanOverflow;			 // 1 if immediate outflow of excess water
double alpha;					 // sqrt(slope)/roughness term in Manning Eq
double alpha_Drain;

int iaIndex;					 // The bin number for the GI 

void GI_Outlets::gi_getRunoff(Inputs* input, DataFolder* folder, int timestep) {

	double Area = folder->ParamDict["Area"] * folder->ParamDict["giNumber"];

	folder->VarDict["test"] = input->RepoDict["infilExRunoff_temp"];

	//	evaluate the GI unit's performance, updating the GIs total surface runoff, drain flow, and flow returned to pervious area 
	evalGI_Unit(folder, Area, timestep);

	folder->VarDict["oldDrainFlow"] = folder->VarDict["newDrainFlow"];
	folder->VarDict["newDrainFlow"] = 0.0;

}

void GI_Outlets::evalGI_Unit(DataFolder* folder, double giArea, int timestep) {

	// Purpose: evaluates performance of a specific GI unit over current time step.
	// Output:
	// qRunoff   = sum of surface runoff from all GIs (m/dt)
	// qDrain    = sum of drain flows from all GIs (m/dt)
	// qReturn   = sum of GI flows returned to pervious area (m/dt)

	double giEvap,			 // evaporation rate from GI unit (m/dt)
		giInfil,             // infiltration rate from GI unit (m/dt)
		giDrain;             // drain flow rate from GI unit (m/dt)

	folder->VarDict["dryTime"] = folder->ParamDict["startDayOfYr"] * SECperDAY;

	// initialize evap and infil losses
	giEvap = 0.0;
	giInfil = 0.0;

	// find surface runoff from the GI unit (m/dt)
	giProc_getOutflow(folder, &giEvap, &giInfil, &giDrain, timestep);

	// convert drain flow to cms
	giDrain *= giArea;

	// update moisture losses (m3)
	Vevap += giEvap * giArea;
	VGiInfil += giInfil * giArea;

	// determines if a GI process allows infiltration or not
	if (folder->ParamDict["storThickness_m"] == 0.0 ||
		folder->ParamDict["seepageRate_mph"] > 0.0) {

		Vpevap += giEvap * giArea;
	}

	// update time since last rainfall (for Rain Barrel emptying)
	if (folder->VarDict["rainfall"] > MIN_RUNOFF) folder->VarDict["dryTime"] = 0.0;
	else folder->VarDict["dryTime"] += folder->ParamDict["tstep"];

	// update GI water balance and save results
	giproc_saveResults(folder);


}
//gi_getRunoff > evalGI_Unit > giProc_getOutflow > modpuls_solve > GIFluxRates (swaleFluxRates) which updates terms in giProc_getOutflow
void GI_Outlets::giProc_getOutflow(DataFolder* folder, double* giEvap,
	double* giInfil, double* giDrain, int timestep) {

	//  Purpose: computes outflows from the surface of a GI unit.
	//  Output:  
	//  GIEvap  = evaporation rate for GI unit (m/dt)
	//	GIInfil = infiltration rate for GI unit (m/dt)
	//  GIDrain = drain flow for GI unit (m/dt)
	//  returns surface runoff rate from the GI unit (m/dt)

	int    i;
	double x[MAX_LAYERS];        // layer moisture levels
	double xOld[MAX_LAYERS];     // work vector
	double xPrev[MAX_LAYERS];    // work vector
	double xMin[MAX_LAYERS];     // lower limit on moisture levels
	double xMax[MAX_LAYERS];     // upper limit on moisture levels
	double fOld[MAX_LAYERS];     // previously computed flux rates
	double f[MAX_LAYERS];        // newly computed flux rates
	// net inflow - outflow from previous time step for each GI layer (m/dt)
	double   oldFluxRates[MAX_LAYERS];

	// convergence tolerance on moisture levels (ft, moisture fraction , ft)
	double xTol[MAX_LAYERS] = { STOPTOL, STOPTOL, STOPTOL, STOPTOL };
	//Li: for keeping ParamDict["storThickness_m"]
	double sthick = 0;


	// rain barrels have 100% void space and impermeable bottom
	if (folder->ParamStringDict["Type"] == "RainBarrel")
	{
		folder->ParamDict["storVoidFrac"] = 1.0;
		folder->ParamDict["seepageRate_mph"] = 0.0;
	}
	// set storage layer parameters of a green roof 
	if (folder->ParamStringDict["Type"] == "GreenRoof")
	{
		folder->ParamDict["storThickness_m"] = folder->ParamDict["drainMatThickness_m"];
		folder->ParamDict["storVoidFrac"] = folder->ParamDict["drainVoidFrac"];
		folder->ParamDict["storClogFactor"] = 0.0;
		folder->ParamDict["seepageRate_mph"] = 0.0;
	}

	if (timestep == 0.0) {
		folder->VarDict["surfaceDepth"] = 0.0;
		folder->VarDict["soilMoisture"] = 0.0;
		folder->VarDict["storageDepth"] = 0.0;
		folder->VarDict["paveDepth"] = 0.0;
		//added bby Li:
		folder->VarDict["total_Evap"]=0.0;

		// set previous flux rates to 0
		for (int i = 0; i < MAX_LAYERS; i++)
		{
			oldFluxRates[i] = 0.0;
		}
	}

	// store current moisture levels in vector x
	x[SURF] = folder->VarDict["surfaceDepth"];
	x[SOIL] = folder->VarDict["soilMoisture"];
	x[STOR] = folder->VarDict["storageDepth"];
	x[PAVE] = folder->VarDict["paveDepth"];

	// initialize layer flux rates and moisture limits
	//SurfaceInflow units of m/dt
	SurfaceInflow = folder->VarDict["giInflow"];
	SurfaceInfil = 0.0;
	SurfaceEvap = 0.0;
	PaveEvap = 0.0;
	PavePerc = 0.0;
	SoilEvap = 0.0;
	SoilPerc = 0.0;
	StorageInflow = 0.0;
	StorageExfil = 0.0;
	StorageEvap = 0.0;
	StorageDrain = 0.0;


	for (i = 0; i < MAX_LAYERS; i++)
	{
		f[i] = 0.0;
		fOld[i] = oldFluxRates[i];
		xMin[i] = 0.0;
		xMax[i] = BIG;
		Xold[i] = x[i];
	}

	//folder->ParamDict["soilThickness_m"] = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	//Li:
	sthick= folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];

	// find Green-Ampt infiltration from surface layer
	if (folder->ParamStringDict["Type"] == "PermeablePavement") 
		SurfaceInfil = 0.0;
	else if (folder->ParamDict["K0_mph"] > 0.0) {
		SurfaceInfil = folder->VarDict["infilExInfil"];
	}
	else SurfaceInfil = folder->VarDict["NativeInfil"];

	//if (folder->ParamDict["soilThickness_m"] > 0.0) {
	//Li:
	if (sthick > 0.0) {
		xMin[SOIL] = folder->ParamDict["soilWiltPoint"];
		xMax[SOIL] = folder->ParamDict["effPorosity"];
	}
	if (folder->ParamDict["paveThickness_m"] > 0.0)
	{
		xMax[PAVE] = folder->ParamDict["paveThickness_m"];
	}
	if (folder->ParamDict["storThickness_m"] > 0.0)
	{
		xMax[STOR] = folder->ParamDict["storThickness_m"];
	}
	if (folder->ParamStringDict["Type"] == "GreenRoof")
	{
		xMax[STOR] = folder->ParamDict["drainMatThickness_m"];
	}

	// determine which flux rate function to use
	// update moisture levels and flux rates over the time step
	modpuls_solve(folder, MAX_LAYERS, x, xOld, xPrev, xMin, xMax, xTol, fOld, f, timestep);



	// save updated results
	folder->VarDict["surfaceDepth"] = x[SURF];
	folder->VarDict["soilMoisture"] = x[SOIL];
	folder->VarDict["storageDepth"] = x[STOR];
	folder->VarDict["paveDepth"] = x[PAVE];

	for (i = 0; i < MAX_LAYERS; i++) oldFluxRates[i] = f[i];

	// assign values to GI unit evaporation, infiltration & drain flow
	*giEvap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	if (*giEvap < 0.0)* giEvap = 0.0;
	*giInfil = StorageExfil;
	*giDrain = StorageDrain;
}


void GI_Outlets::modpuls_solve(DataFolder* folder, int n, double* x, double* xOld, double* xPrev,
	double* xMin, double* xMax, double* xTol, double* qOld, double* q, double dt) {

	//  Purpose: solves system of equations dx/dt = q(x) for x at end of time step dt using a modified Puls method.
	int i;
	// initialize state variable values
	for (i = 0; i < n; i++)
	{
		xOld[i] = x[i];
		xPrev[i] = x[i];
	}

	// compute flux rates for current state levels
	if (folder->ParamStringDict["Type"] == "BioRetention") {
		bioretentionFluxRates(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "RainGarden") {
		//Raingarden is a special case of bioretention where storThickness_m = 0 
		bioretentionFluxRates(folder, x, q);

	}
	if (folder->ParamStringDict["Type"] == "GreenRoof") {
		greenRoofFluxRates(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "InfilTrench") {
		trenchFluxRates(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "PermeablePavement") {
		pavementFluxRates(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "RainBarrel") {
		barrelFluxRates(folder, x, q);
	}
	//Li:
	if (folder->ParamStringDict["Type"] == "RoofDiscon") {
		roofFluxRates(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "Swale") {
		swaleFluxRates(folder, x, q);
	}

	// update state levels based on current flux rates, 
	// i represents layers [0 = surface, 1 = layer below; see Figure 5.8 from dissertation of Reza Abdi]
	for (i = 0; i < n; i++)
	{
		//update water balance, find min and max, reset negative value to 0; q[i] is equal to f[] in modPuls function
		//for most GI, q[0] = f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow)
		//for swale GI q[0] = f[SURF] = dVdt/surfArea and dVdT = surfInflow - SurfaceEvap - SoilPerc - SurfaceOutflow
		x[i] = xOld[i] + q[i];
		x[i] = MIN(x[i], xMax[i]);
		x[i] = MAX(x[i], xMin[i]);
		if (x[i] < 0.0) { x[i] = 0.0; }
			   
	}
	folder->VarDict["waterToPerDep"] = x[0];

}

//bioretentionFluxRates is used by rainGardenFluxRates; it is a special case where storThickness_m = 0
void GI_Outlets::bioretentionFluxRates(DataFolder* folder, double x[], double f[]) {

	//  Purpose: computes flux rates from the layers of a bio-retention cell GI.

	// Moisture level variables
	double surfaceDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;

	// GI layer properties
	// Hydro based equation
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	double effPorosity = folder->ParamDict["effPorosity"];
	double soilFieldCap = folder->ParamDict["soilFieldCapacity"];
	double soilWiltPoint = folder->ParamDict["soilWiltPoint"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];

	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];
	//Added by Li:
	double total_Evap;
	
	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	// Convert moisture levels to volumes
	//SurfaceVolume = surfaceDepth * (1.0 - folder->ParamDict["surfaceVoidRatio"]);
	//SoilVolume = soilTheta * soilThickness;
	//StorageVolume = storageDepth * storageVoidFrac;
	// Get ET rates using SWMM functions; this could be used if Hydro ET is not wanted
	//availVolume = SoilVolume - soilWiltPoint * soilThickness;
	//getEvapRates(folder, SurfaceVolume, 0.0, availVolume, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	SoilEvap = folder->VarVecDict["rET_TI"][folder->ParamDict["binNumber"]];
	StorageEvap = 0.0;   // No storage evap based on the calculations of the Hydro
	if (soilTheta >= effPorosity) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// soil layer perc rate
	// A new rootZoneQ_TI created for GI is defined by percolation function  
	double rootZoneQ_TI = getSoilPercRate(folder, soilTheta);
	SoilPerc = rootZoneQ_TI;

	// updating the rooz zone Q for the GI
	// This set of TI bins is within the GI area, and we assume they have the same distribution as the catchment area, a fractal relationship
	for (int ia = 0; ia < 30; ia++) {
		folder->VarVecDict["rootZoneQ_TI"][ia] = SoilPerc;
	}

	// exfiltration rate out of storage layer
	StorageExfil = getStorageExfilRate(folder);

	folder->VarDict["upSat_2"] = 0.0;	// Two parameters showing the differences between SWMM & Hydro layers in RZ & USZ
	folder->VarDict["upSat_3"] = 0.0;
	folder->VarDict["satZoneQ"] = 0.0;	// The available water in sat zone of the GI which should go to BA


		// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		StorageDrain = getStorageDrainRate(folder, storageDepth, soilTheta, 0.0,
			surfaceDepth);
	}

	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the storage layer
	if (folder->VarDict["Zi"] < 0.0) { folder->VarDict["Zi"] = 0.0; }
	if (folder->VarDict["Zi"] >= soilThickness + storageThickness) {
		//SoilPerc = folder->VarVecDict["rootZoneQ_TI"][folder->ParamDict["binNumber"]];
		// This is an incremental Average SMD which will be added to the total AveSMD of the bulk area
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil *
			(folder->VarDict["Zi"] / (soilThickness + storageThickness)));
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = (StorageExfil * (folder->VarDict["Zi"] / (soilThickness + storageThickness)));
	}
	// Zi is within storage layer 
	if ((folder->VarDict["Zi"] > soilThickness) && (folder->VarDict["Zi"] <= (storageThickness + soilThickness - folder->ParamDict["drainOffset"]))) {
		//The AveSMD_GI is increased by StorageDrain flow and decreased by StorageExfil; AveSMD for BulkArea is udpated with AveSMD_GI
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX(((soilThickness + storageThickness -
			folder->VarDict["Zi"]) * folder->ParamDict["storVoidFrac"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}
	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] < soilThickness) && (folder->VarDict["Zi"] >= 0.0)) {
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["effPorosity"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}

	// limit perc rate by available water
	availVolume = (soilTheta - soilFieldCap) * soilThickness;
	maxRate = MAX(availVolume, 0.0) - SoilEvap;
	SoilPerc = MIN(SoilPerc, maxRate);
	SoilPerc = MAX(SoilPerc, 0.0);

	// special case of no storage layer present
	if (storageThickness == 0.0)
	{
		StorageEvap = 0.0;
		maxRate = MIN(SoilPerc, StorageExfil);
		SoilPerc = maxRate;
		StorageExfil = maxRate;

		// limit surface infil. by unused soil volume
		maxRate = (effPorosity - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);

	}

	// storage & soil layers are full
	else if (soilTheta >= effPorosity && storageDepth >= storageThickness)
	{
		// limiting rate is smaller of soil perc and storage outflow
		maxRate = StorageExfil + StorageDrain;
		if (SoilPerc < maxRate)
		{
			maxRate = SoilPerc;
			if (maxRate > StorageExfil) StorageDrain = maxRate - StorageExfil;
			else
			{
				StorageExfil = maxRate;
				StorageDrain = 0.0;
			}
		}
		else SoilPerc = maxRate;

		// apply limiting rate to surface infil.
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// either layer not full
	else if (storageThickness > 0.0)
	{
		// limit storage exfiltration by available storage volume
		maxRate = SoilPerc - StorageEvap + storageDepth * storageVoidFrac;
		StorageExfil = MIN(StorageExfil, maxRate);
		StorageExfil = MAX(StorageExfil, 0.0);

		// limit underdrain flow by volume above drain offset
		if (StorageDrain > 0.0)
		{
			maxRate = -StorageExfil - StorageEvap;
			if (storageDepth >= storageThickness) maxRate += SoilPerc;
			if (folder->ParamDict["drainOffset"] <= storageDepth)
			{
				maxRate += (storageDepth - (folder->ParamDict["drainOffset"] *
					storageVoidFrac));

			}
			maxRate = MAX(maxRate, 0.0);
			StorageDrain = MIN(StorageDrain, maxRate);
		}

		// limit soil perc by unused storage volume
		maxRate = StorageExfil + StorageDrain + StorageEvap +
			(storageThickness - storageDepth) * storageVoidFrac;
		SoilPerc = MIN(SoilPerc, maxRate);

		// limit surface infil. by unused soil volume
		maxRate = (effPorosity - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}
	// saving the exfilteration
	folder->VarDict["infilToSoil"] = StorageExfil;

	// Surface outflow in this section was occuring at the end of the time step for each device (bioretention, raingarden, permeable pavement, infiltration trench)
	// Commented out to allow for surface outflow to be called at the beginning of the time step, to minimize ponding and impacts on infiltration
	// Leave these commented lines until additional model testing confirms they can be deleted - August 2019
	//SurfaceOutflow = getSurfaceOutflowRate(folder, surfaceDepth);

	// compute overall layer flux rates
	// the value within that time step, represents infilExRunoff when qo > f1
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow) /
		folder->ParamDict["surfaceVoidRatio"];

	//this is the change in theta2 not d2
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;
	
	if (storageThickness == 0.0) f[STOR] = 0.0;
	else f[STOR] = (SoilPerc - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
		folder->ParamDict["storVoidFrac"];
	
}

void GI_Outlets::greenRoofFluxRates(DataFolder* folder, double x[], double f[]) {

	//
	//  Purpose: computes flux rates from the layers of a green roof.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	 // Moisture level variables
	double surfaceDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;


	// Green roof properties
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	double effPorosity = folder->ParamDict["effPorosity"];
	double soilFieldCap = folder->ParamDict["soilFieldCapacity"];
	double soilWiltPoint = folder->ParamDict["soilWiltPoint"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	//Added by Li:
	double total_Evap;

	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];

	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	//SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	//SoilVolume = soilTheta * soilThickness;
	//StorageVolume = storageDepth * storageVoidFrac;
	// get ET rates
	//availVolume = SoilVolume - soilWiltPoint * soilThickness;
	//getEvapRates(folder, SurfaceVolume, 0.0, availVolume, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	SoilEvap = folder->VarVecDict["rET_TI"][0];
	StorageEvap = 0.0;   // No storage evap based on the calculations of the Hydro
	if (soilTheta >= effPorosity) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// soil layer perc rate
	SoilPerc = getSoilPercRate(folder, soilTheta);

	// limit perc rate by available water
	availVolume = (soilTheta - soilFieldCap) * soilThickness;
	maxRate = MAX(availVolume, 0.0) - SoilEvap;
	SoilPerc = MIN(SoilPerc, maxRate);
	SoilPerc = MAX(SoilPerc, 0.0);

	// storage (drain mat) outflow rate
	StorageExfil = 0.0;
	StorageDrain = getDrainMatOutflow(folder, storageDepth);

	// unit is full
	if (soilTheta >= effPorosity && storageDepth >= storageThickness)
	{
		// outflow from both layers equals limiting rate
		maxRate = MIN(SoilPerc, StorageDrain);
		SoilPerc = maxRate;
		StorageDrain = maxRate;

		// adjust inflow rate to soil layer
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// unit not full
	else
	{
		// limit drainmat outflow by available storage volume
		maxRate = storageDepth * storageVoidFrac - StorageEvap;
		if (storageDepth >= storageThickness) maxRate += SoilPerc;
		maxRate = MAX(maxRate, 0.0);
		StorageDrain = MIN(StorageDrain, maxRate);

		// limit soil perc inflow by unused storage volume
		maxRate = (storageThickness - storageDepth) * storageVoidFrac +
			StorageDrain + StorageEvap;
		SoilPerc = MIN(SoilPerc, maxRate);

		// adjust surface infil. so soil porosity not exceeded
		maxRate = (effPorosity - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}
	

	// compute overall layer flux rates
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow) /
		folder->ParamDict["surfaceVoidRatio"];
	//f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) /
		//folder->ParamDict["effPorosity"];
	//Li:
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;

	f[STOR] = (SoilPerc - StorageEvap - StorageDrain) /
		folder->ParamDict["storVoidFrac"];

}

void GI_Outlets::trenchFluxRates(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates from the layers of an infiltration trench GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	 // Moisture level variables
	double surfaceDepth;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;

	// Storage layer properties
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	double soilThickness = 0.0; // should be 0
	//Added by Li:
	double total_Evap;


	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	storageDepth = x[STOR];

	// convert moisture levels to volumes
	//SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	//SoilVolume = 0.0;
	//StorageVolume = storageDepth * storageVoidFrac;
	// get ET rates
	//availVolume = (storageThickness - storageDepth) * storageVoidFrac;
	//getEvapRates(folder, SurfaceVolume, 0.0, 0.0, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	StorageEvap = SoilEvap = folder->VarVecDict["rET_TI"][folder->ParamDict["binNumber"]];
	// no storage evap if surface ponded
	if (surfaceDepth > 0.0) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// nominal storage inflow
	StorageInflow = SurfaceInflow + SurfaceVolume;

	// exfiltration rate out of storage layer
	StorageExfil = getStorageExfilRate(folder);
	//StorageExfil = folder->VarDict["unsatZoneQ"];	// f3 in the manual

	// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		StorageDrain = getStorageDrainRate(folder, storageDepth, 0.0, 0.0, surfaceDepth);
	}

	// parameters showing the differences between SWMM & Hydro layers in RZ & USZ
	folder->VarDict["upSat_3"] = 0.0;
	folder->VarDict["satZoneQ"] = 0.0;	// The available water in sat zone of the GI which should go to BA


	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the storage layer
	if (folder->VarDict["Zi"] < 0.0) { folder->VarDict["Zi"] = 0.0; }
	if (folder->VarDict["Zi"] >= storageThickness) {
		//SoilPerc = folder->VarVecDict["rootZoneQ_TI"][folder->ParamDict["binNumber"]];
		// This is an incremental Average SMD which will be added to the total AveSMD of the bulk area
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil *
			(folder->VarDict["Zi"] / (storageThickness)));
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = (StorageExfil * (folder->VarDict["Zi"] / (storageThickness)));
	}
	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] < storageThickness) && (folder->VarDict["Zi"] >= 0.0)) {
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil;
		folder->VarDict["upSat_3"] = MAX(((storageThickness - folder->VarDict["Zi"]) * folder->ParamDict["storVoidFrac"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}
	else if (folder->VarDict["Zi"] < 0.0) {
		StorageExfil = folder->VarDict["unsatZoneQ"];
		StorageExfil = folder->VarVecDict["rootZoneQ_TI"][folder->ParamDict["binNumber"]];
		folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["effPorosity"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}

	// limit storage exfiltration by available storage volume
	maxRate = StorageInflow - StorageEvap + storageDepth * storageVoidFrac;
	StorageExfil = MIN(StorageExfil, maxRate);
	StorageExfil = MAX(StorageExfil, 0.0);

	// limit underdrain flow by volume above drain offset
	if (StorageDrain > 0.0)
	{
		maxRate = -StorageExfil - StorageEvap;
		if (storageDepth >= storageThickness) maxRate += StorageInflow;
		if (folder->ParamDict["drainOffset"] <= storageDepth)
		{
			maxRate += (storageDepth - folder->ParamDict["drainOffset"]) *
				storageVoidFrac;
		}
		maxRate = MAX(maxRate, 0.0);
		StorageDrain = MIN(StorageDrain, maxRate);
	}

	// limit storage inflow to not exceed storage layer capacity
	maxRate = (storageThickness - storageDepth) * storageVoidFrac +
		StorageExfil + StorageEvap + StorageDrain;
	StorageInflow = MIN(StorageInflow, maxRate);

	// equate surface infil to storage inflow
	SurfaceInfil = StorageInflow;

	// find net fluxes for each layer
	f[SURF] = SurfaceInflow - SurfaceEvap - StorageInflow - SurfaceOutflow /
		folder->ParamDict["surfaceVoidRatio"];

	f[STOR] = (StorageInflow - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
		folder->ParamDict["storVoidFrac"];

	f[SOIL] = 0.0;
}

void GI_Outlets::pavementFluxRates(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for the layers of a porous pavement GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	// Moisture level variables
	double surfaceDepth;
	double paveDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double pervFrac = (1.0 - folder->ParamDict["pavementImpervFrac"]);
	double storageInflow;    // inflow rate to storage layer (ft/s)
	double availVolume;
	double maxRate;

	// GI layer properties
	double paveVoidFrac = folder->ParamDict["pavementVoidFrac"] * pervFrac;
	double paveThickness = folder->ParamDict["paveThickness_m"];
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	double effPorosity = folder->ParamDict["effPorosity"];
	double soilFieldCap = folder->ParamDict["soilFieldCapacity"];
	double soilWiltPoint = folder->ParamDict["soilWiltPoint"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	//Added by Li:
	double total_Evap;


	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	paveDepth = x[PAVE];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];

	// convert moisture levels to volumes
	SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	PaveVolume = paveDepth * paveVoidFrac;
	SoilVolume = soilTheta * soilThickness;
	StorageVolume = storageDepth * storageVoidFrac;

	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	// get ET rates
	//availVolume = SoilVolume - soilWiltPoint * soilThickness;
	//getEvapRates(folder, SurfaceVolume, PaveVolume, availVolume, StorageVolume, pervFrac);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	PaveEvap = folder->VarVecDict["rET_TI"][0];
	StorageEvap = 0.0;

	// no storage evap if soil or pavement layer saturated
	if (paveDepth >= paveThickness ||
		(soilThickness > 0.0 && soilTheta >= effPorosity)
		) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// find nominal rate of surface infiltration into pavement layer
	SurfaceInfil = SurfaceInflow + (SurfaceVolume);

	// find perc rate out of pavement layer
	PavePerc = getPavementPermRate(folder);
	
	// surface infiltration can't exceed pavement permeability              //(5.1.013)
	SurfaceInfil = MIN(SurfaceInfil, PavePerc);                                //

	// limit pavement perc by available water
	maxRate = PaveVolume + SurfaceInfil - PaveEvap;
	maxRate = MAX(maxRate, 0.0);
	PavePerc = MIN(PavePerc, maxRate);

	// find soil layer perc rate
	if (soilThickness > 0.0)
	{
		SoilPerc = getSoilPercRate(folder, soilTheta);
		availVolume = (soilTheta - soilFieldCap) * soilThickness;
		maxRate = MAX(availVolume, 0.0) - SoilEvap;
		SoilPerc = MIN(SoilPerc, maxRate);
		SoilPerc = MAX(SoilPerc, 0.0);
	}
	else SoilPerc = PavePerc;

	// exfiltration rate out of storage layer
	StorageExfil = getStorageExfilRate(folder);


	// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		StorageDrain = getStorageDrainRate(folder, storageDepth, soilTheta, paveDepth,
			surfaceDepth);
	}

	folder->VarDict["upSat_2"] = 0.0;	// Two parameters showing the differences between SWMM & Hydro layers in RZ & USZ
	folder->VarDict["upSat_3"] = 0.0;
	folder->VarDict["upSat_4"] = 0.0;
	folder->VarDict["satZoneQ"] = 0.0;	// The available water in sat zone of the GI which should go to BA


	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the storage layer
	if (folder->VarDict["Zi"] < 0.0) { folder->VarDict["Zi"] = 0.0; }
	if (folder->VarDict["Zi"] >= soilThickness + storageThickness + paveThickness) {
		// This is an incremental Average SMD which will be added to the total AveSMD of the bulk area
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil *
			(folder->VarDict["Zi"] / (soilThickness + storageThickness)));
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = (StorageExfil * (folder->VarDict["Zi"] / (soilThickness + storageThickness)));
	}
	// Zi is within storage layer 
	if ((folder->VarDict["Zi"] > (soilThickness + paveThickness)) && (folder->VarDict["Zi"] <= (storageThickness + soilThickness + paveThickness - folder->ParamDict["drainOffset"]))) {

		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX(((soilThickness + storageThickness + paveThickness -
			folder->VarDict["Zi"]) * folder->ParamDict["storVoidFrac"]), 0.0);

		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}
	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] > paveThickness) && (folder->VarDict["Zi"] <= (soilThickness + paveThickness))) {
		//StorageExfil = folder->VarDict["unsatZoneQ"];
		//SoilPerc = folder->VarVecDict["rootZoneQ_TI"][folder->ParamDict["binNumber"]];
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		folder->VarDict["upSat_2"] = MAX(((soilThickness + paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["effPorosity"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
	}
	// Zi is within pave layer 
	else if ((folder->VarDict["Zi"] > 0) && (folder->VarDict["Zi"] <= paveThickness)) {

		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		folder->VarDict["upSat_2"] = MAX(((soilThickness + paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["effPorosity"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = StorageExfil;
		//New
		folder->VarDict["upSat_4"] = MAX(((paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["pavementVoidFrac"]), 0.0);

	}

	// check for adjacent saturated layers

	// no soil layer, pavement & storage layers are full
	if (soilThickness == 0.0 &&
		storageDepth >= storageThickness &&
		paveDepth >= paveThickness)
	{
		// pavement outflow can't exceed storage outflow
		maxRate = StorageEvap + StorageDrain + StorageExfil;
		if (PavePerc > maxRate) PavePerc = maxRate;

		// storage outflow can't exceed pavement outflow
		else
		{
			// use up available exfiltration capacity first
			StorageExfil = MIN(StorageExfil, PavePerc);
			StorageDrain = PavePerc - StorageExfil;
		}

		// set soil perc to pavement perc
		SoilPerc = PavePerc;

		// limit surface infil. by pavement perc
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// pavement, soil & storage layers are full
	else if (soilThickness > 0 &&
		storageDepth >= storageThickness &&
		soilTheta >= effPorosity &&
		paveDepth >= paveThickness)
	{
		// find which layer has limiting flux rate
		maxRate = StorageExfil + StorageDrain;
		if (SoilPerc < maxRate) maxRate = SoilPerc;
		else maxRate = MIN(maxRate, PavePerc);

		// use up available storage exfiltration capacity first
		if (maxRate > StorageExfil) StorageDrain = maxRate - StorageExfil;
		else
		{
			StorageExfil = maxRate;
			StorageDrain = 0.0;
		}
		SoilPerc = maxRate;
		PavePerc = maxRate;

		// limit surface infil. by pavement perc
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// storage & soil layers are full
	else if (soilThickness > 0.0 &&
		storageDepth >= storageThickness &&
		soilTheta >= effPorosity)
	{
		// soil perc can't exceed storage outflow
		maxRate = StorageDrain + StorageExfil;
		if (SoilPerc > maxRate) SoilPerc = maxRate;

		// storage outflow can't exceed soil perc
		else
		{
			// use up available exfiltration capacity first
			StorageExfil = MIN(StorageExfil, SoilPerc);
			StorageDrain = SoilPerc - StorageExfil;
		}

		// limit surface infil. by available pavement volume
		availVolume = (paveThickness - paveDepth) * paveVoidFrac;
		maxRate = availVolume + PavePerc + PaveEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// soil and pavement layers are full
	else if (soilThickness > 0.0 &&
		paveDepth >= paveThickness &&
		soilTheta >= effPorosity)
	{
		PavePerc = MIN(PavePerc, SoilPerc);
		SoilPerc = PavePerc;
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// no adjoining layers are full
	else
	{
		// limit storage exfiltration by available storage volume
		//    (if no soil layer, SoilPerc is same as PavePerc)
		maxRate = SoilPerc - StorageEvap + StorageVolume;
		maxRate = MAX(0.0, maxRate);
		StorageExfil = MIN(StorageExfil, maxRate);

		// limit underdrain flow by volume above drain offset
		if (StorageDrain > 0.0)
		{
			maxRate = -StorageExfil - StorageEvap;
			if (storageDepth >= storageThickness) maxRate += SoilPerc;
			if (folder->ParamDict["drainOffset"] <= storageDepth)
			{
				maxRate += (storageDepth - folder->ParamDict["drainOffset"]) *
					storageVoidFrac;
			}
			maxRate = MAX(maxRate, 0.0);
			StorageDrain = MIN(StorageDrain, maxRate);
		}

		// limit soil & pavement outflow by unused storage volume
		availVolume = (storageThickness - storageDepth) * storageVoidFrac;
		maxRate = availVolume + StorageEvap + StorageDrain + StorageExfil;
		maxRate = MAX(maxRate, 0.0);
		if (soilThickness > 0.0)
		{
			SoilPerc = MIN(SoilPerc, maxRate);
			maxRate = (effPorosity - soilTheta) * soilThickness +
				SoilPerc;
		}
		PavePerc = MIN(PavePerc, maxRate);

		// limit surface infil. by available pavement volume
		availVolume = (paveThickness - paveDepth) * paveVoidFrac;
		maxRate = availVolume + PavePerc + PaveEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// Copying the pavePerc here for the flowIntoRootZone.cpp to replace with flowIntoRootZone only for the PermeablePavement
	folder->VarDict["pavePerc"] = PavePerc;


	// surface outflow
	//SurfaceOutflow = getSurfaceOutflowRate(folder, surfaceDepth);

	// compute overall layer flux rates
	f[SURF] = SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow;
	f[PAVE] = (SurfaceInfil - PaveEvap - PavePerc + folder->VarDict["upSat_4"]) /
		paveVoidFrac;

	// This next conditional is equivalent to that used by SWMM, if ( theLidProc->soil.thickness > 0.0), so use "if MSD_m > 0" 
	if (folder->ParamDict["MSD_m"] > 0.0)
	{
		f[SOIL] = (PavePerc - SoilEvap - SoilPerc + folder->VarDict["upSat_2"]) /
			folder->ParamDict["effPorosity"];
		storageInflow = SoilPerc;
	}
	else
	{
		f[SOIL] = 0.0;
		storageInflow = PavePerc;
		SoilPerc = 0.0;
	}
	f[STOR] = (storageInflow - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
		storageVoidFrac;


}

void GI_Outlets::roofFluxRates(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for roof disconnection.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	double surfaceDepth = x[SURF];
	//Added by Li:
	double total_Evap;

	//getEvapRates(folder, surfaceDepth, 0.0, 0.0, 0.0, 1.0);
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	//Added by Li:
	total_Evap = SurfaceEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	SurfaceVolume = surfaceDepth;
	SurfaceInfil = 0.0;
	if (alpha > 0.0)
		SurfaceOutflow = getSurfaceOutflowRate(folder, surfaceDepth);
	else getSurfaceOverflowRate(folder, surfaceDepth);
	StorageDrain = MIN(folder->ParamDict["drainCoef"] / folder->VarDict["waterOnImperviousArea"], SurfaceOutflow);
	SurfaceOutflow -= StorageDrain;
	f[SURF] = (SurfaceInflow - SurfaceEvap - StorageDrain - SurfaceOutflow);
	
}

void GI_Outlets::barrelFluxRates(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for a rain barrel GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	double storageDepth = x[STOR];
	double head;
	double maxValue;
	//Added by Li:
	double total_Evap;

	// assign values to layer volumes
	SurfaceVolume = 0.0;
	SoilVolume = 0.0;
	StorageVolume = storageDepth;

	// initialize flows
	SurfaceInfil = 0.0;
	SurfaceOutflow = 0.0;
	StorageDrain = 0.0;

	// compute outflow if time since last rain exceeds drain delay
	//    (dryTime is updated in lid.evalLidUnit at each time step)
	if (folder->ParamDict["drainDelay"] == 0.0 ||
		folder->VarDict["dryTime"] >= (folder->ParamDict["drainDelay"] * 60.0))
	{
		head = storageDepth - folder->ParamDict["drainOffset"];
		if (head > 0.0)
		{
			StorageDrain = getStorageDrainRate(folder, storageDepth, 0.0, 0.0, 0.0);
			maxValue = (head);
			StorageDrain = MIN(StorageDrain, maxValue);
		}
	}

	//Added by Li:
	total_Evap = 0;
	folder->VarDict["total_Evap"] = total_Evap;

	// limit inflow to available storage
	StorageInflow = SurfaceInflow;
	maxValue = (folder->ParamDict["storThickness_m"] - storageDepth) +
		StorageDrain;
	StorageInflow = MIN(StorageInflow, maxValue);
	SurfaceInfil = StorageInflow;

	// assign values to layer flux rates
	f[SURF] = SurfaceInflow - StorageInflow;
	f[STOR] = StorageInflow - StorageDrain;
	f[SOIL] = 0.0;

}


void GI_Outlets::swaleFluxRates(DataFolder* folder, double x[], double f[]) {
	//  Purpose: computes flux rates from a vegetative swale GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	double depth;            // depth of surface water in swale (m)
	double topWidth;         // top width of full swale (m)
	double botWidth;         // bottom width of swale (m)
	double length;           // length of swale (m)
	double surfInflow;       // inflow rate to swale (m/dt)
	double surfWidth;        // top width at current water depth (m)
	double surfArea;         // surface area of current water depth (m2)
	double flowArea;         // x-section flow area (m2)
	double giArea;          // surface area of full swale (m2)
	double hydRadius;        // hydraulic radius for current depth (fmt)
	double slope;            // slope of swale side wall (run/rise)
	double volume;           // swale volume at current water depth (m3)
	double dVdT;             // change in volume w.r.t. time (m/dt)
	double dStore;           // depression storage depth (m)
	double xDepth;           // depth above depression storage (m)
	double soilTheta;		 // soil volumetric moisture (m^3/m^3)
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"]; // soil thickness (m) = (maximum storage depth for water) / (effective porosity)
	double soilFieldCap = folder->ParamDict["soilFieldCapacity"]; // soil field capacity (m^3/m^3)
	double total_Evap; // total evaporation to maintain water balance (m)
	double maxRate; // term used to find rate limiting step in water balance algorithm
	double storageThickness = folder->ParamDict["storThickness_m"]; // storage layer below the soil, optional part of device
	double effPorosity = folder->ParamDict["effPorosity"]; // porosity or less, if parts of porosity not available


	// retrieve state variable from work vector
	depth = x[SURF];
	// added the soil moisture in Swale; not simulated by SWMM
	soilTheta = x[SOIL];

	depth = MIN(depth, folder->ParamDict["surfaceBermH_m"]);

	// depression storage depth
	dStore = 0.0;

	// get swale's bottom width
	slope = folder->ParamDict["surfaceSideSlope"];
	topWidth = folder->ParamDict["giWidthAtOutlet_m"];
	// set to 0.5 m minimum to avoid numerical problems
	topWidth = MAX(topWidth, 0.5);
	//Eq from Chin (2013) figure 5.2
	botWidth = topWidth - 2.0 * slope * folder->ParamDict["surfaceBermH_m"];
	// set to 0.25 m minimum to avoid numerical problems
	if (botWidth < 0.25)
	{
		botWidth = 0.25;
		slope = 0.5 * (topWidth - 0.25) / folder->ParamDict["surfaceBermH_m"];
	}

	// swale's length
	giArea = folder->ParamDict["Area"];
	length = giArea / topWidth;

	// top width, surface area and flow area of current ponded depth
	surfWidth = botWidth + 2.0 * slope * depth;
	surfArea = length * surfWidth;
	//Eq 5.3 from Chin (2013) A = b*y + m*y^2, where b is bottom width, y is water depth, m is side slope
	flowArea = (depth * (botWidth + slope * depth)) *
		folder->ParamDict["surfaceVoidRatio"];

	// volume is actual water volume within swale, wet volume and effective depth
	volume = length * flowArea;

	// surface inflow into swale (m/dt) * giArea (m^2) = q0 (m^3/dt)
	surfInflow = SurfaceInflow * giArea;

	//Creating SurfaceEvap term (m) for water budget of device
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];

	//SurfaceEvap is minimum of SurfaceEvap demand or available water depth (volume/giArea) to get depth per time
	SurfaceEvap = MIN(SurfaceEvap, (volume/giArea) / folder->ParamDict["tstep"]);

	//Creating soil evaporation for water balance
	SoilEvap = folder->VarVecDict["rET_TI"][0];
	//Summing total evaporation for water balance
	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// Soil layer perc rate
	SoilPerc = getSoilPercRate(folder, soilTheta);
	// updating the rooz zone Q for the GI
	for (int ia = 0; ia < 30; ia++) {
		folder->VarVecDict["rootZoneQ_TI"][ia] = SoilPerc;
	}

	// Exfiltration rate to native soil 
	StorageExfil = getStorageExfilRate(folder);

	// Special case of no storage layer below soil, typical for swale, rainGarden and similar devices
	if (storageThickness == 0.0)
	{
		StorageEvap = 0.0;
		//maxRate is minimum of SoilPerc or StorageExfil, as rate limiting step
		maxRate = MIN(SoilPerc, StorageExfil);
		SoilPerc = maxRate;
		StorageExfil = maxRate;

		// limit surface infil. by unused soil volume
		maxRate = (effPorosity - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}
	//Setting to 0 the parameter for upwelling saturation from rising watertable  
	folder->VarDict["upSat_2"] = 0.0;	// the parameter showing the differences between SWMM & Hydro layers in RZ & USZ

	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the soil layer
	if (folder->VarDict["Zi"] < 0.0) { folder->VarDict["Zi"] = 0.0; }
	if (folder->VarDict["Zi"] >= soilThickness) {
		//SoilPerc = folder->VarVecDict["rootZoneQ_TI"][folder->ParamDict["binNumber"]];
		// This is an incremental Average SMD which will be added to the total AveSMD of the bulk area
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (SoilPerc *
			(folder->VarDict["Zi"] / (soilThickness)));
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = (SoilPerc * (folder->VarDict["Zi"] / (soilThickness)));
	}
	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] < soilThickness) && (folder->VarDict["Zi"] >= 0.0)) {
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - SoilPerc;
		folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["effPorosity"]), 0.0);
		// updating the unsatZoneQ
		folder->VarDict["unsatZoneQ"] = SoilPerc;
	}
	// Intermediate variables
	double availVolume;
	double maxRate_Swale;
	double wettetPerimeter = 0;
	// no surface outflow if depth below depression storage
	xDepth = depth - dStore;
	if (xDepth <= ZERO) SurfaceOutflow = 0.0;

	// otherwise compute a surface outflow
	else
	{
		// modify flow area to remove depression storage,
		flowArea -= (dStore * (botWidth + slope * dStore)) *
			folder->ParamDict["surfaceVoidRatio"];
		if (flowArea < ZERO) SurfaceOutflow = 0.0;
		else
		{
			// compute hydraulic radius
			botWidth = botWidth + 2.0 * dStore * slope; //cte - 2020 June 9 I question this equation, need to look into why bottom width updated by dStore
			//Eq 5.4 from Chin (2013) P = b + 2*y(1+m^2)^0.5 where b is bottom width, y is water depth, m is side slope
			wettetPerimeter = botWidth + 2.0 * xDepth * sqrt(1.0 + slope * slope);
			hydRadius = flowArea / wettetPerimeter;

			// use Manning Eqn. to find outflow rate units of cms or m^3/second
			SurfaceOutflow = alpha * flowArea *
				pow(hydRadius, (2.0 / 3.0));

			//convert from m^3/s to m^3/dt for comparison with other volume per time variables, dV/dt
			SurfaceOutflow *= folder->ParamDict["tstep"];
			SurfaceOutflow /= giArea;

		}
	}
	
	// net water balance (m)
	//UH correctly computes balance with area-adjusted depths, unlike SWMM code
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow);
	
	// when full, any net positive inflow becomes spillage
	// SurfaceBermH_m term is used in place of the SWMM term surface.thickness; they are equivalent
	if (depth >= folder->ParamDict["surfaceBermH_m"] && f[SURF] > 0.0)
	{
		//SWMM erroneously tried to update balance: SurfaceOutflow += dVdT;
		//SWMM erroneously tried to update balance: dVdT = 0.0;
		//UG correctly updates balance
		SurfaceOutflow += f[SURF];
		f[SURF] = 0.0;
	}

	// express soil moisture as per soil thickness, as volumetric moisture
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;
	f[STOR] = 0.0;

	// assign values to layer volumes
	SurfaceVolume = volume / giArea;
	SoilVolume = 0.0;
	StorageVolume = 0.0;

	/*SWMM code commented out and moved; eronneously tried to update water balance
	//StorageExfil = SurfaceInfil * surfArea;
	//dVdT = surfInflow - SurfaceEvap - SoilPerc - SurfaceOutflow;
	//StorageExfil /= giArea;
	//SoilPerc /= giArea;
	//SurfaceOutflow /= giArea;
	//f[SURF] = dVdT / surfArea;
	//f[SOIL] = 0.0;
	*/


}


//========================
//============================================================================
//============================================================================
void GI_Outlets::getEvapRates(DataFolder* folder, double surfaceVol, double paveVol, double soilVol,
	double storageVol, double pervFrac) {

	//  Purpose: computes surface, pavement, soil, and storage evaporation rates.
	//  Input:   
	// surfaceVol = volume/area of ponded water on surface layer (m)
	// paveVol    = volume/area of water in pavement pores (m)
	// soilVol    = volume/area of water in soil (or pavement) pores (m)
	// storageVol = volume/area of water in storage layer (m)
	// pervFrac   = fraction of surface layer that is pervious

	double availEvap;

	// surface evaporation flux
	availEvap = EvapRate; // m/dt
	SurfaceEvap = MIN(availEvap, surfaceVol);
	SurfaceEvap = MAX(0.0, SurfaceEvap);
	availEvap = MAX(0.0, (availEvap - SurfaceEvap));
	availEvap *= pervFrac;

	// no subsurface evap if water is infiltrating
	if (SurfaceInfil > 0.0)
	{
		PaveEvap = 0.0;
		SoilEvap = 0.0;
		StorageEvap = 0.0;
	}
	else
	{
		// pavement evaporation flux
		PaveEvap = MIN(availEvap, paveVol);
		availEvap = MAX(0.0, (availEvap - PaveEvap));

		// soil evaporation flux
		SoilEvap = MIN(availEvap, soilVol);
		availEvap = MAX(0.0, (availEvap - SoilEvap));

		// storage evaporation flux
		StorageEvap = MIN(availEvap, storageVol);
	}

}

//=============================================================================
double GI_Outlets::getSoilPercRate(DataFolder* folder, double theta) {

	//  Purpose: computes percolation rate of water through a GI's soil layer.
	//  Input:   theta = moisture content (fraction)
	//  Output:  returns percolation rate within soil layer

	double delta;            // moisture deficit
	double perc;
	// no percolation if soil moisture <= field 
	if (theta <= folder->ParamDict["soilFieldCapacity"]) return 0.0;

	// perc rate = unsaturated hydraulic conductivity
	delta = folder->ParamDict["effPorosity"] - theta;

	//Equation comes from SWMM code for LID processes
	perc = folder->ParamDict["K0_mph"] * exp(-delta * folder->ParamDict["soilCondSlope"]);

	//Equation comes from Abdi and Endreny to limit percolation to length of water available for gravitational drainage
	//	theta * soilFieldCapacity gives available fractional water, MSD_m * effPorosity gives length of soil
	if (perc > (theta - folder->ParamDict["soilFieldCapacity"]) *
		(folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"])) {
		perc = (theta - folder->ParamDict["soilFieldCapacity"]) * ((folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"]));
	}
	else {
		perc = folder->ParamDict["K0_mph"] * exp(-delta * folder->ParamDict["soilCondSlope"]);
	}


	return perc;
}

//============================================================================
double GI_Outlets::getStorageExfilRate(DataFolder* folder) {

	//  Purpose: computes exfiltration rate from storage zone into
	//           native soil beneath a GI.
	//  Input:   depth = depth of water storage zone (m)
	//  Output:  returns exfiltration rate (mph) called infil

	double exfil = 0.0;
	double clogFactor = 0.0;

	if (folder->ParamDict["seepageRate_mph"] == 0.0) return 0.0;
	if (folder->VarDict["MaxNativeExfil"] == 0.0) return 0.0;

	// reduction due to clogging
	clogFactor = folder->ParamDict["storClogFactor"];
	if (clogFactor > 0.0)
	{
		clogFactor = folder->VarDict["waterBalanceInflow"] / clogFactor;
		clogFactor = MIN(clogFactor, 1.0);
	}

	// infiltration rate = storage storage conductivity (seepage rate) reduced by any clogging
	exfil = folder->ParamDict["seepageRate_mph"] * (1.0 - clogFactor);

	// limit infiltration rate by any groundwater-imposed limit
	return MIN(exfil, folder->VarDict["MaxNativeExfil"]);
}

//=============================================================================
double GI_Outlets::getStorageDrainRate(DataFolder* folder, double storageDepth,
	double soilTheta, double paveDepth, double surfaceDepth) {

	//  Purpose: computes underdrain flow rate in a GI's storage layer.
	//  Input:   storageDepth = depth of water in storage layer (m)
	//           soilTheta    = moisture content of soil layer
	//           paveDepth    = effective depth of water in pavement layer (m)
	//           surfaceDepth = depth of ponded water on surface layer (m)
	//  Output:  returns flow in underdrain (m/s)

	// Abdi: Note storage depth can sharply decreases due to storage drainage >> percolation in prior time step.
	// Abdi: Adjust drainOrificeQ_coef variable to solve the problem of the rapid variations in drainage. 

	double head = storageDepth;
	double outflow = 0.0;
	double paveThickness = folder->ParamDict["paveThickness_m"];
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	double effPorosity = folder->ParamDict["effPorosity"];
	double soilFieldCap = folder->ParamDict["soilFieldCapacity"];
	double storageThickness = folder->ParamDict["storThickness_m"];

	// Assess thickness of layers to determine which layers are saturated storage layer is full
	if (storageDepth >= storageThickness)
	{
		// a soil layer exists
		if (soilThickness > 0.0)
		{
			// increase head by fraction of soil layer saturated
			if (soilTheta > soilFieldCap)
			{
				head += (soilTheta - soilFieldCap) /
					(effPorosity - soilFieldCap) * soilThickness;

				// soil layer is saturated, increase head by water
				//     depth in layer above it
				if (soilTheta >= effPorosity)
				{
					if (paveThickness > 0.0) head += paveDepth;
					else head += surfaceDepth;
				}
			}
		}

		// no soil layer so increase head by water level in pavement layer and possibly surface layer
		if (paveThickness > 0.0)
		{
			head += paveDepth;
			if (paveDepth >= paveThickness) head += surfaceDepth;
		}
	}

	folder->VarDict["head"] = head;


	// make head relative to drain offset
	// offset is the distance from the bottom of the storage layer to the bottom of the pipe (units meters)
	head -= folder->ParamDict["drainOffset"];

	// compute drain outflow from underdrain flow equation 
	if (head > ZERO)
	{
		head /= (0.3048); // to conver to ft

		// This equation is in the SWMM BG units (ft), and will be converted to SI (meters) per time step 
		// See how EPA SWMM interface allwos user input choice; Drain Advisor from SWMM interface
		//  compute drain outflow in user ft/timestep
		outflow = folder->ParamDict["drainOrificeQ_coef"] *
			pow(head, folder->ParamDict["drainOrificeQ_exponent"]);

		// Convert to meters/timestep
		outflow *= (0.3048); 

	}
	folder->VarDict["newDrainFlow"] = outflow;

	return outflow;
}

//=============================================================================
// Computing outflow rate using some undefined form of Manning Equation
double GI_Outlets::getSurfaceOutflowRate(DataFolder* folder, double depth) {

	//  Purpose: computes outflow rate from a GI's surface layer.
	//  Input:   depth = depth of ponded water on surface layer (m)
	//  Output:  returns outflow from surface layer (m)

	//  Note: this function should not be applied to swales or rain barrels.

	double delta;
	double outflow;


	// no outflow if ponded depth below storage depth of berm crest
	delta = depth - folder->ParamDict["surfaceBermH_m"];

	if (delta < 0.0) return 0.0;

	// Compute outflow from overland flow Manning equation from SWMM code (theory needed)
	// Based on Eq 3.3 of SWMM Ref Manual Vol I - Hydrology (2016), 
	// Derivation:	Volumetric flow = Q = 1/n * Width * Depth^(5/3) * Slope^(1/2)
	//				Q (m^3/s) is function of Manning velocity and channel cross section area
	//				XS Area = Axs = Width * Depth
	//				Hydraulic Radius = R = Depth 
	//				R^(2/3) * Axs^(3/3) = Width * Depth^(5/3)
	//				alpha = 1/n * Slope^(1/2)
	//				delta = Depth = effective runoff depth
	//				Q (m^3/s) divided by basin area is runoff depth across basin
	outflow = alpha * pow(delta, 5.0 / 3.0) *
		folder->ParamDict["giWidthAtOutlet_m"] / folder->ParamDict["Area"];
	//SWMM output is in m/s, so SWMM outflow was not multiplied by tstep, and SWMM delta was divided by tstep
	outflow = MIN(outflow * folder->ParamDict["tstep"], delta);
	return outflow;
}

//=============================================================================
// Based on emergency spillway to reset depth to berm height and excess water is sent to overflow 
double GI_Outlets::getSurfaceOverflowRate(DataFolder* folder, double surfaceDepth) {

	//  Purpose: finds surface overflow rate from a GI unit.
	//  Input:   surfaceDepth = depth of water stored in surface layer (m)
	//  Output:  returns the overflow rate (m)

	double delta = surfaceDepth - folder->ParamDict["surfaceEmergencyH_m"];

	if (delta <= 0.0) return 0.0;
	surfaceDepth = folder->ParamDict["surfaceBermH_m"];
	return delta * folder->ParamDict["storVoidFrac"];
}

//=============================================================================
// Internal function to save GI results to output writer
void GI_Outlets::giproc_saveResults(DataFolder* folder) {

	//  Purpose: updates the mass balance for a GI unit and saves
	//  current flux rates to the GI report file.

	double totalEvap;                  // total evaporation rate (m/s)
	double totalVolume;                // total volume stored in GI (m)
	int    isDry = FALSE;              // true if current state of GI is dry
	double elapsedHrs;                 // elapsed hours

	// find total evaporation rate and stored volume
	totalEvap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	if (totalEvap < 0.0) { totalEvap = 0; }
	totalVolume = SurfaceVolume + PaveVolume + SoilVolume + StorageVolume;

	// update mass balance totals writing to WaterBalance_[GI].csv
	if (folder->ParamStringDict["Type"] == "PermeablePavement") {
		updateWaterBalancePermeablePavement(folder, SurfaceInflow, SurfaceInfil, SoilPerc, StorageExfil,
			SurfaceOutflow, StorageDrain, folder->VarDict["paveDepth"],
			folder->VarDict["surfaceDepth"], folder->VarDict["soilMoisture"],
			folder->VarDict["storageDepth"], folder->VarDict["pavePerc"], folder->VarDict["total_Evap"]);
	}
	//Edited by Li:
	//1. Added the argument of total Evap
	//2. To use updateWaterBalance for all GIs except PermeablePavement
	//3. To change the argument from folder->VarDict["test"] to folder->VarDict["storageDepth"] to fix the d3 problem in output file
	//VarDict["test"]= folder->VarDict["infilExRunoff"]= runoff-infiltrated flow, the storage depth(d3) shouldn't be this value.

	//else if ((folder->ParamStringDict["Type"] == "RainGarden")|| (folder->ParamStringDict["Type"] == "RainBarrel") || (folder->ParamStringDict["Type"] == "GreenRoof")) {
	//	updateWaterBalance(folder, SurfaceInflow, SurfaceInfil, SoilPerc, StorageExfil,
	//		SurfaceOutflow, StorageDrain, folder->VarDict["paveDepth"],
	//		folder->VarDict["surfaceDepth"], folder->VarDict["soilMoisture"],
	//		folder->VarDict["storageDepth"], folder->VarDict["total_Evap"]);
	//}

	else {
		updateWaterBalance(folder, SurfaceInflow, SurfaceInfil, SoilPerc, StorageExfil,
			SurfaceOutflow, StorageDrain, folder->VarDict["paveDepth"], 
			folder->VarDict["surfaceDepth"],folder->VarDict["soilMoisture"],
			folder->VarDict["storageDepth"], folder->VarDict["total_Evap"]);
	}

	// check if dry-weather conditions hold
	if (SurfaceInflow < MINFLOW &&
		SurfaceOutflow < MINFLOW &&
		StorageDrain < MINFLOW &&
		StorageExfil < MINFLOW &&
		totalEvap < MINFLOW
		) isDry = TRUE;
}

//=============================================================================
// Providing vectors for output writer, generic to all GI but Permeable Pavement (which has unique variable pavePerc_out)
void GI_Outlets::updateWaterBalance(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
	double surfFlow, double drainFlow, double paveDepth, double surfaceDepth, double soilMoisture,
	double storageDepth,double total_Evap) {

	//  Purpose: updates components of the water mass balance for a GI unit
	//  over the current time step.

	folder->VarDict["waterBalanceInflow"] = inflow;
	folder->VarDict["waterBalanceInfil"] = infil;
	folder->VarDict["soilPerc"] = soilPerc;
	folder->VarDict["waterBalanceExfil"] = exfil;
	folder->VarDict["waterBalanceSurFlow"] = surfFlow;
	folder->VarDict["waterBalanceDrainFlow"] = drainFlow;
	folder->VarDict["paveDepth_out"] = paveDepth;
	folder->VarDict["surfaceDepth_out"] = surfaceDepth;
	folder->VarDict["soilMoisture_out"] = soilMoisture;
	folder->VarDict["storageDepth_out"] = storageDepth;
	//Added by Li to show the total Evap
	folder->VarDict["evaporation_all"] = total_Evap;

	folder->VarVecDict["waterBalanceInflow"].emplace_back(folder->VarDict["waterBalanceInflow"]);
	folder->VarVecDict["waterBalanceInfil"].emplace_back(folder->VarDict["waterBalanceInfil"]);
	folder->VarVecDict["soilPerc"].emplace_back(folder->VarDict["soilPerc"]);
	folder->VarVecDict["waterBalanceExfil"].emplace_back(folder->VarDict["waterBalanceExfil"]);
	folder->VarVecDict["waterBalanceSurFlow"].emplace_back(folder->VarDict["waterBalanceSurFlow"]);
	folder->VarVecDict["waterBalanceDrainFlow"].emplace_back(folder->VarDict["waterBalanceDrainFlow"]);
	folder->VarVecDict["paveDepth_out"].emplace_back(folder->VarDict["paveDepth_out"]);
	folder->VarVecDict["surfaceDepth_out"].emplace_back(folder->VarDict["surfaceDepth_out"]);
	folder->VarVecDict["soilMoisture_out"].emplace_back(folder->VarDict["soilMoisture_out"]);
	folder->VarVecDict["storageDepth_out"].emplace_back(folder->VarDict["storageDepth_out"]);
	//Added by Li to show the total Evap
	folder->VarVecDict["evaporation_all"].emplace_back(folder->VarDict["evaporation_all"]);
	
}

//=============================================================================
// Providing vectors for output writer, specific to Permeable Pavement which has unique variable pavePerc_out
void GI_Outlets::updateWaterBalancePermeablePavement(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
	double surfFlow, double drainFlow, double paveDepth, double surfaceDepth, double soilMoisture,
	double storageDepth, double pavePerc, double total_Evap) {

	//  Purpose: updates components of the water mass balance for a GI unit
	//  over the current time step.

	folder->VarDict["waterBalanceInflow"] = inflow;
	folder->VarDict["waterBalanceInfil"] = infil;
	folder->VarDict["soilPerc"] = soilPerc;
	folder->VarDict["waterBalanceExfil"] = exfil;
	folder->VarDict["waterBalanceSurFlow"] = surfFlow;
	folder->VarDict["waterBalanceDrainFlow"] = drainFlow;
	folder->VarDict["paveDepth_out"] = paveDepth;
	folder->VarDict["surfaceDepth_out"] = surfaceDepth;
	folder->VarDict["soilMoisture_out"] = soilMoisture;
	folder->VarDict["storageDepth_out"] = storageDepth;
	folder->VarDict["pavePerc_out"] = pavePerc;
	//Added by Li to show the total Evap
	folder->VarDict["evaporation_all"] = total_Evap;
	
	folder->VarVecDict["waterBalanceInflow"].emplace_back(folder->VarDict["waterBalanceInflow"]);
	folder->VarVecDict["waterBalanceInfil"].emplace_back(folder->VarDict["waterBalanceInfil"]);
	folder->VarVecDict["soilPerc"].emplace_back(folder->VarDict["soilPerc"]);
	folder->VarVecDict["waterBalanceExfil"].emplace_back(folder->VarDict["waterBalanceExfil"]);
	folder->VarVecDict["waterBalanceSurFlow"].emplace_back(folder->VarDict["waterBalanceSurFlow"]);
	folder->VarVecDict["waterBalanceDrainFlow"].emplace_back(folder->VarDict["WaterBalanceDrainFlow"]);
	folder->VarVecDict["paveDepth_out"].emplace_back(folder->VarDict["paveDepth_out"]);
	folder->VarVecDict["surfaceDepth_out"].emplace_back(folder->VarDict["surfaceDepth_out"]);
	folder->VarVecDict["soilMoisture_out"].emplace_back(folder->VarDict["soilMoisture_out"]);
	folder->VarVecDict["storageDepth_out"].emplace_back(folder->VarDict["storageDepth_out"]);
	folder->VarVecDict["pavePerc_out"].emplace_back(folder->VarDict["pavePerc_out"]);
	//Added by Li to show the total Evap
	folder->VarVecDict["evaporation_all"].emplace_back(folder->VarDict["evaporation_all"]);
}


//=============================================================================
//Called from TestSimulation at beginning of time step, after bulkArea complete, to initiate the GI vectors and variables 
void GI_Outlets::giproc_initWaterBalance(DataFolder* folder) {

	//  Purpose: initializes the water balance components of a GI unit.

	double initVol = 0.0;
	double soilMoisture = 0.0;
	double storageDepth = 0.0;
	//Li: for keeping ParamDict["soilThickness_m"]
	double sthick = 0.0;
	//folder->ParamDict["soilThickness_m"] = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	//Li:
	sthick = folder->ParamDict["MSD_m"] / folder->ParamDict["effPorosity"];
	//defining the initial GI soil saturation based on the SD0 parameter, which is saturated when = 0
	folder->ParamDict["initiallySat_frac"] = (1.0 - (folder->ParamDict["SD0_percent"] / 100.0));

	/*if (folder->ParamDict["soilThickness_m"] > 0.0)
	{
		soilMoisture = folder->ParamDict["soilThickness_m"] +
			folder->ParamDict["initiallySat_frac"] * (folder->ParamDict["effPorosity"] -
				folder->ParamDict["soilWiltPoint"]);
		initVol += soilMoisture * folder->ParamDict["soilThickness_m"];
	}*/
	//Li:
	if (sthick > 0.0)
	{
		soilMoisture = sthick +
			folder->ParamDict["initiallySat_frac"] * (folder->ParamDict["effPorosity"] -
				folder->ParamDict["soilWiltPoint"]);
		initVol += soilMoisture * sthick;
	}
	if (folder->ParamDict["storThickness_m"] > 0.0)
	{
		storageDepth = folder->ParamDict["initiallySat_frac"] *
			folder->ParamDict["storThickness_m"];
		initVol += storageDepth * folder->ParamDict["storVoidFrac"];
	}
	if (folder->ParamDict["drainMatThickness_m"] > 0.0)
	{
		storageDepth = folder->ParamDict["initiallySat_frac"] *
			folder->ParamDict["drainMatThickness_m"];
		initVol += storageDepth * folder->ParamDict["drainMatThickness_m"];
	}

	folder->VarDict["VolTreated"] = 0.0;
	folder->VarDict["waterBalanceInflow"] = 0.0;
	folder->VarDict["waterBalanceEvap"] = 0.0;
	folder->VarDict["waterBalanceInfil"] = 0.0;
	folder->VarDict["waterBalanceSurFlow"] = 0.0;
	folder->VarDict["waterBalanceDrainFlow"] = 0.0;
	folder->VarDict["waterBalanceInitVol"] = initVol;
	folder->VarDict["waterBalanceFinalVol"] = initVol;

	folder->VarDict["waterToPerDep_GI"] = 0.0;
	folder->VarDict["waterOnImperviousArea_GI"] = 0.0;

	//iaIndex = folder->ParamDict["iaIndex"];
	
}

//Called from TestSimulation later in time step, after evaporation, to continue with estimating outflow
void GI_Outlets::getGI_Outflow(Inputs* input, DataFolder* folder, int timestep) {
	//
	//  Purpose: computes runoff and drain flows from the GIs for the available data folders.
	//  Output:  updates following global quantities after GI applied:
	//           Vevap, Vpevap, VGiInfil, VlidIn, VlidOut, VlidDrain.
	//


	double giArea;               // area of a GI unit
	double qImperv = 0.0;         // runoff from impervious areas (m/dt)
	double qPerv = 0.0;           // runoff from pervious areas (m/dt)          
	double giInflow = 0.0;       // inflow to a GI unit (m/dt) 
	double qRunoff = 0.0;         // surface runoff from all GI units (m/dt)
	double qDrain = 0.0;          // drain flow from all GI units (m/dt)
	double qReturn = 0.0;         // GI outflow returned to pervious area (m/dt)
	SurfaceOutflow = 0.0;

	// for certain GI types, immediate overflow of excess surface water
	// occurs if either the surface roughness or slope is zero
	surfaceCanOverflow = TRUE;
	if (folder->ParamStringDict["Type"] == "RoofDiscon") {
		surfaceCanOverflow = FALSE;
	}

	if (folder->ParamDict["surfaceRoughness"] > 0.0) {
		if (folder->ParamStringDict["Type"] == "Swale") {
			alpha = sqrt(folder->ParamDict["surfaceSlope_frac"]) / folder->ParamDict["surfaceRoughness"];
		}
		else {
			alpha = sqrt(folder->ParamDict["surfaceSlope_frac"]) / folder->ParamDict["surfaceRoughness"];
		}

	}
	else alpha = 0.0;
	//Li: Just for green roof to compute drainage mat layer's flow coeff
	//Changed the code according to SWMM algorithm
	/* SWMM:
	if (LidProcs[j].drainMat.roughness > 0.0)
	{
		LidProcs[j].drainMat.alpha = 1.49 / LidProcs[j].drainMat.roughness *
			sqrt(LidProcs[j].surface.surfSlope);*/

	/*Reza: 
	if (folder->ParamDict["drainRoughness"] > 0.0) {
		alpha_Drain = folder->ParamDict["drainRoughness"] / sqrt(folder->ParamDict["surfaceSlope_frac"]);
	}*/

	if (folder->ParamDict["drainRoughness"] > 0.0) {
		alpha_Drain = sqrt(folder->ParamDict["surfaceSlope_frac"]) / folder->ParamDict["drainRoughness"];
	}

	else alpha_Drain = 0.0;
	//Li :Added for RoofDiscon
	if ((folder->ParamStringDict["Type"] == "RainGarden") || (folder->ParamStringDict["Type"] == "BioRetention") ||
		(folder->ParamStringDict["Type"] == "GreenRoof") || (folder->ParamStringDict["Type"] == "RainBarrel") ||
		(folder->ParamStringDict["Type"] == "PermeablePavement") || (folder->ParamStringDict["Type"] == "Swale") ||
		(folder->ParamStringDict["Type"] == "InfilTrench") || (folder->ParamStringDict["Type"] == "RoofDiscon")) {
		if (alpha > 0) {
			surfaceCanOverflow = FALSE;
		}
	}

	// total drain flow in previous period, will be updated for each time step
	folder->VarDict["MaxNativeExfil"] = 0.0;

	// return if there are no GI
	if (folder->ParamDict["giNumber"] == 0) return;

	// evaporation initializing
	EvapRate = folder->VarDict["totalEvap"];  //(m/dt)


	// infiltration rate into soil
	// tstep is the time step and is in seconds
	folder->VarDict["NativeInfil"] = folder->VarDict["infilToSoil"];
	// MaxNativeInfil depends on the groundwater in the basin,
	// if we have groundwater (in future updates) we need to set the
	// MaxNativeInfil value to the Max infil. rate of the basin considering
	// the groundwater divided by the time step (sec)

	folder->VarDict["MaxNativeExfil"] = BIG;          // Generic large value

	// find area of the GI unit
	double Area = folder->ParamDict["Area"] * folder->ParamDict["giNumber"];

	// pervious and impervious flows of the BulkArea; the input-> is from model layer above folder
	folder->VarDict["PAQ"] = input->RepoDict["PAQ"];
	folder->VarDict["DCIAQ"] = input->RepoDict["DCIAQ"];


	// evaluate performance of each GI unit placed in the subcatchment
	// if the GI unit has area, evaluate its performance
	if (folder->ParamDict["Area"] != 0.0) {

		//This is setting giInflow to the BulkArea waterToPerDep_pre and GI waterOnImperviousArea and precipitation (see Storage > WaterOnImpArea and WaterOnPerArea)
		giInflow = folder->VarDict["waterToPerDep_pre"] + folder->VarDict["waterOnImperviousArea"]; // PAQ + DCIAQ + Rainfall

		folder->VarDict["giInflow"] = giInflow;

		// Surface outflow calculated for all GI except RainBarrel and Swale, which use different functions
		if (folder->ParamStringDict["Type"] != "RainBarrel") {
			if (folder->ParamStringDict["Type"] != "Swale") {

				// Calculate the surface outflow, q1, based on the surface depth relative to berm height
				SurfaceOutflow = getSurfaceOutflowRate(folder, folder->VarDict["surfaceDepth"]);
			}
		}
	}
	
	// add any surface overflow to surface outflow
	if (surfaceCanOverflow || folder->ParamDict["giWidthAtOutlet_m"] == 0.0)
	{
		// Add emergency spillway water to SurfaceOutflow
		SurfaceOutflow += getSurfaceOverflowRate(folder, folder->VarDict["surfaceDepth"]);
	}

	if (SurfaceOutflow < 0.0) { SurfaceOutflow = 0.0; }

	// updating the waterONPer/Ipm for the GI 

	if (folder->VarDict["waterOnImperviousArea"] < 0) {
		folder->VarDict["waterOnImperviousArea"] = 0.0;
	}
	if (folder->VarDict["waterToPerDep"] < 0) {
		folder->VarDict["waterToPerDep"] = 0.0;
	}
	if (folder->VarDict["waterOnPerviousArea"] < 0) {
		folder->VarDict["waterOnPerviousArea"] = 0.0;
	}
}

double GI_Outlets::getDrainMatOutflow(DataFolder* folder, double depth)
//
//  Purpose: computes flow rate through a green roof's drainage mat.
//  Input:   depth = depth of water in drainage mat (m)
//  Output:  returns flow in drainage mat (m/s)
//
{
	//... default is to pass all inflow
	double result = SoilPerc;

	//... otherwise use Manning eqn. if its parameters were supplied
	if (alpha_Drain > 0.0)
	{
		result = alpha_Drain * pow(depth, 5.0 / 3.0) *
			folder->ParamDict["giWidthAtOutlet_m"] / folder->ParamDict["Area"] *
			folder->ParamDict["drainVoidFrac"];
	}
	return result;
}

// cte: This regeneration option is currently not active; uncomment and test against SWMM code if needed 
//=============================================================================
double GI_Outlets::getPavementPermRate(DataFolder* folder) {

	//
	//  Purpose: computes reduced permeability of a pavement layer due to
	//           clogging.
	//  Input:   none
	//  Output:  returns the reduced permeability of the pavement layer.
	//

	double permReduction = 0.0;
	double clogFactor = folder->ParamDict["paveClogFactor"];
	double regenDays = folder->ParamDict["paveRegenDays"];

	// find permeability reduction due to clogging     
	if (clogFactor > 0.0)
	{
		//  see if permeability regeneration has occurred (regeneration is assumed to reduce the total volumetric loading that the pavement has received)
		if (regenDays > 0.0)
		{
			
			//if (OldRunoffTime / 1000.0 / SECperDAY >= theLidUnit->nextRegenDay)
			//{
			//	// ... reduce total volume treated by degree of regeneration
			//	theLidUnit->volTreated *=
			//		(1.0 - theLidProc->pavement.regenDegree);

			//	// ... update next day that regenration occurs
			//	theLidUnit->nextRegenDay += regenDays;
			//}

		}

		// find permeabiity reduction factor
		//permReduction = theLidUnit->volTreated / clogFactor;
		//permReduction = MIN(permReduction, 1.0);
	}

	// return the effective pavement permeability
	return (folder->ParamDict["pavePermeability_mph"] * (1.0 - permReduction)) / folder->VarDict["rainfall"];
}

//=============================================================================

