///////////////////////////////////////////////////////////////
//  Pollution.h   -  Calculating various pollution data      //
//  ver 1.0                                                  //
//                                                           //
//  Language:     Visual C++, ver 6.0                        //
//  Application:  Pollution part for Hydro                   //
//  Author:       Rahul Kumbhar, Syracuse Univ.              //
//                (617)416-3561, rahul.d.kumbhar@gmail.com   //
///////////////////////////////////////////////////////////////

#include "Pollution.h"
#include <iomanip>
#include <string>
#include <fstream>

using namespace std;

Pollution::Pollution(double timesteps, double catchmentarea, std::string directory) : timesteps(timesteps), catchmentarea(catchmentarea), OutputDirLocation(directory)  
{

	std::ifstream infile(OutputDirLocation + "Pollutiondata.dat");
	if (!infile.good())
	{
		PollutionDataFileExists = false;
		return;
	}
	else
	{
		PollutionDataFileExists = true;
	}
	std::string type;
	infile>>type>>type>>type>>type>>type>>type>>type>>type>>type>>type>>type;
	
	infile>>type>>TssmedDensity>>BODmedDensity>>CODmedDensity>>TPmedDensity>>SolPmedDensity>>TKNmedDensity>>NO2_3medDensity>>CumedDensity>>PbmedDensity>>ZnmedDensity;

	infile>>type>>TssmeanDensity>>BODmeanDensity>>CODmeanDensity>>TPmeanDensity>>SolPmeanDensity>>TKNmeanDensity>>NO2_3meanDensity>>CumeanDensity>>PbmeanDensity>>ZnmeanDensity;

	
	infile.close();
	

}



void Pollution::calPollutionData(std::vector<double> &Per, std::vector<double> &ImpPer)
{
	double landflow = 0;
	double volume = 0;
	TreePervious = Per;
	TreeImpervious = ImpPer;
	try{
	for(int i=0; i<timesteps; ++i)
	{
		//cte - check if landflow should include NoTreeImpervious, ShortVeg, Soil. Define units of volume and density
		landflow = TreePervious[i] + TreeImpervious[i];
		volume = landflow * catchmentarea * 1000;
		calMass(Tssmedmass_kg, volume, TssmedDensity);
		calMass(Tssmeanmass_kg, volume,TssmeanDensity);
		calMass(BODmedmass_kg, volume, BODmedDensity);
		calMass(BODmeanmass_kg, volume,BODmeanDensity);
		calMass(CODmedmass_kg, volume, CODmedDensity);
		calMass(CODmeanmass_kg, volume, CODmeanDensity);
		calMass(TPmedmass_kg, volume, TPmedDensity);
		calMass(TPmeanmass_kg, volume, TPmeanDensity);
		calMass(SolPmedmass_kg, volume, SolPmedDensity);
		calMass(SolPmeanmass_kg, volume, SolPmeanDensity);
		calMass(TKNmedmass_kg, volume, TKNmedDensity);
		calMass(TKNmeanmass_kg, volume, TKNmeanDensity);
		calMass(NO2_3medmass_kg, volume, NO2_3medDensity);
		calMass(NO2_3meanmass_kg, volume, NO2_3meanDensity);
		calMass(Cumedmass_kg, volume, CumedDensity);
		calMass(Cumeanmass_kg, volume, CumeanDensity);
		calMass(Pbmedmass_kg, volume, PbmedDensity);
		calMass(Pbmeanmass_kg, volume, PbmeanDensity);
		calMass(Znmedmass_kg, volume, ZnmedDensity);
		calMass(Znmeanmass_kg, volume, ZnmeanDensity);
	}
	} catch(std::exception& ex)
    {
      std::cout << "  " << ex.what() << "\n\n";
    }
}

void Pollution::calMass(std::vector<double> &mass, double volume, double med_mean)
{
	mass.push_back((volume * med_mean)/1000000);
}

void Pollution::writePollutiondata(std::vector<double> &date, std::vector<double> &time, std::string OutputDir)
{
	

	std::ofstream outfile(OutputDir + "Pollution.dat");
	if(!outfile.good())
		return;

	outfile <<"  "<<std::setw(5)  <<"Time Steps"
			//<<"  "<<std::setw(5)  <<"YYYYMMDD"
			//<<"  "<<std::setw(10) <<"Hr:min:sec"
			<<"  "<<std::setw(10) <<"Tssmedmass(kg)"
            <<"  "<<std::setw(10) <<"Tssmeanmass(kg)"
            <<"  "<<std::setw(10) <<"BODmedmass(kg)"
            <<"  "<<std::setw(10) <<"BODmeanmass(kg)"
			<<"  "<<std::setw(13) <<"CODmedmass(kg)"
			<<"  "<<std::setw(13) <<"CODmeanmass(kg)"
			<<"  "<<std::setw(10) <<"TPmedmass(kg)"
            <<"  "<<std::setw(10) <<"TPmeanmass(kg)"
			<<"  "<<std::setw(13) <<"SolPmedmass(kg)"
			<<"  "<<std::setw(13) <<"SolPmeanmass(kg)"
			<<"  "<<std::setw(10) <<"TKNmedmass(kg)"
            <<"  "<<std::setw(10) <<"TKNmeanmass(kg)"
			<<"  "<<std::setw(13) <<"NO2_3medmass(kg)"
			<<"  "<<std::setw(13) <<"NO2_3meanmass(kg)"
			<<"  "<<std::setw(10) <<"Cumedmass(kg)"
            <<"  "<<std::setw(10) <<"Cumeanmass(kg)"
			<<"  "<<std::setw(13) <<"Pbmedmass(kg)"
			<<"  "<<std::setw(13) <<"Pbmeanmass(kg)"
			<<"  "<<std::setw(10) <<"Znmedmass(kg)"
            <<"  "<<std::setw(10) <<"Znmeanmass(kg)"
			
			<<endl<<endl;

	for ( int i=0; i<timesteps; i++)
	{
		outfile <<"  "<<std::setw(5) <<i+1
				//<<"  "<<std::setw(5) <<date[i]
			    //<<"  "<<std::setw(10)<<time[i]
				<<"  "<<std::setw(10)<<Tssmedmass_kg[i]
                <<"  "<<std::setw(10)<<Tssmeanmass_kg[i]
                <<"  "<<std::setw(10)<<BODmedmass_kg[i]
                <<"  "<<std::setw(10)<<BODmeanmass_kg[i]
				<<"  "<<std::setw(13)<<CODmedmass_kg[i]
				<<"  "<<std::setw(13)<<CODmeanmass_kg[i]
				<<"  "<<std::setw(13)<<TPmedmass_kg[i]
                <<"  "<<std::setw(13)<<TPmeanmass_kg[i]
				<<"  "<<std::setw(10)<<SolPmedmass_kg[i]
                <<"  "<<std::setw(10)<<SolPmeanmass_kg[i]
				<<"  "<<std::setw(13)<<TKNmedmass_kg[i]
				<<"  "<<std::setw(13)<<TKNmeanmass_kg[i]
				<<"  "<<std::setw(13)<<NO2_3medmass_kg[i]
                <<"  "<<std::setw(13)<<NO2_3meanmass_kg[i]
				<<"  "<<std::setw(10)<<Cumedmass_kg[i]
                <<"  "<<std::setw(10)<<Cumeanmass_kg[i]
				<<"  "<<std::setw(13)<<Pbmedmass_kg[i]
				<<"  "<<std::setw(13)<<Pbmeanmass_kg[i]
				<<"  "<<std::setw(13)<<Znmedmass_kg[i]
                <<"  "<<std::setw(13)<<Znmeanmass_kg[i]
				<< endl;
	}

	outfile.close();
}



#ifdef POLLUTION_TEST

void main() {


}

#endif