#ifndef RCALIBRATE_H
#define RCALIBRATE_H

/////////////////////////////////////////////////////////////////////////////
//																		   //
// Calibrate.h - Model calibration from runoff              	           //																		   
// ver 1.0                                                                 //
// Language:    Visual C++ 2010                                            //
// Platform:    MacBookPro5,4  Win7										   //
// Application: HydroPrime/iHydro Version6, Summer 2012                    //
// Programmer:  				   //																	   	
/////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: Version 4.5 & Version 5 
  
  Source Code File(s): calibrate.h, rcalibrate.h, rcalibrate.cpp
  
  Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author 
		     Yang Yang, yyang31@syr.edu 
						*Added VE variable and double getVE() function  
			  	
						*Eliminated inheritance by combining calibrate rcalibrate classes
						*Adjusted public interface to have two constructors that read either
							the v4 Qobs.dat [calibrate(std::string directory)] or v5 Qobs.dat file [calibrate(std::string directory, double startDate, double endDate)]
						*Changed size calculation in the funciton objFunRunoff(), line 130
  Creation Date: Summer 2000

  Publication(s): Jun Wang PHDThesis

  Module Operations: 
  ==================
  ...

  Public Interface:
  =================
        calibrate(std::string directory);
		calibrate(std::string directory, double startDate, double endDate);
		int size();
		void objFunRunoff(std::vector<double>& );
		double getF1();                    // get CRF1
        double getF2();                    // get CRF2
		double getF3();                    // get CRF3
		double getE();                     // get SE
		double getVE();
		std::vector<double>& getQobs();
		double operator[] (size_t i) const;    // for observed runoff

  Build Process:
  ==============
  Required files
	-Qobs.dat
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================

 */

#include "calibrate.h"
#include <vector>
#include <string>
class calibrate 
{
	private:
		// VE added by Yang 
		double VE;
		double CRF1;         // 1-SUM(Qobsi-Qsim)^2/SUM(Qobsi-aveQobs)^2
		double CRF2;         // 1-SUM|Qobsi-Qsim|/SUM|Qobsi-aveQobs|
		double CRF3;         // 1-SUM(sqrt(Qobsi)-sqrt(Qsim))^2/SUM(sqrt(Qobsi)-sqrt(aveQobs))^2
		double SE;           // mean square error
		std::vector<double>  _Qobs;

		// Addition of Timestep change code by Pallavi Iyengar
		std::vector <double> dailyQobs;  // the summation of 24 hours Q
		std::vector <double> weeklyQobs; // the summation of 7 days Q
		// End of Timestep changes by Pallavi Iyengar
		
		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector <double> monthlyQobs; // the summation of 30 days of Q
		// End of Monthly Timestep changes by Isira Samarasekera

	public: 
		calibrate(std::string directory);
		calibrate(std::string directory, double startDate, double endDate);
		
		// Addition of Timestep change code by Pallavi Iyengar
		calibrate(std::vector<double>& Qsim, std::vector<double>& Qobs);
		// End of Timestep changes by Pallavi Iyengar

		int size();
		void objFunRunoff(std::vector<double>& );
		double getF1();                    // get CRF1
        double getF2();                    // get CRF2
		double getF3();                    // get CRF3
		double getE();                     // get SE
		//added by Yang
		double getVE();
		std::vector<double>& getQobs();
		double operator[] (size_t i) const;    // for observed runoff
	   
		// Addition of Timestep change code by Pallavi Iyengar
		inline std::vector<double>& getDailyQobs(){ return dailyQobs; };
		inline std::vector<double>& getWeeklyQobs(){ return weeklyQobs; };
		void calcDailyQobs();
		void calcWeeklyQobs();
		// End of Timestep changes by Pallavi Iyengar

		//  Addition of Monthly Timestep change code by Isira Samarasekera
		inline std::vector<double>& getMonthlyQobs(){ return monthlyQobs; };

		void calcMonthlyQobs();
		// End of Monthly Timestep changes by Isira Samarasekera

};
//--

//------------< get zise>-----------------------------------------//

inline int calibrate::size() { return _Qobs.size(); }

//--

//------------< get CRF1>-----------------------------------------//

inline double calibrate::getF1() { return CRF1; }

//--

//------------< get CRF2>-----------------------------------------//

inline double calibrate::getF2() { return CRF2; }

//--

//------------< get CRF3>-----------------------------------------//

inline double calibrate::getF3() { return CRF3; }

//--
//------------< get SE>-----------------------------------------//

inline double calibrate::getE() { return SE; }

//--

//-------< get observed time series runoff  >------------------------------------

inline std::vector<double>& calibrate::getQobs() { return _Qobs;}

inline double calibrate::getVE() { return VE; }

//--

#endif