#include "RootZoneETCalc.h"

 
void RootZoneETCalc::calculateRootzoneETStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	int ia = input->RepoDict["iA"];
	double rzMSD = folder->ParamDict["MSD_m"];
	// if potential ET (PET - read in from Evaporation.dat input file) > 0 
	if (input->PET[timeStep] > 0.0)
	{
		// Modify rootzone ET to equal = PET * (1 - current rootzone storage deficit / maximum rootzone storage deficit), for this index value
		// the more saturated the rootzone, the lower and closer to 0 the rootzone storage deficit is, and the great the actual ET
		// if rootzone storage deficit = 0, the actualt ET = PET
		folder->VarVecDict["rET_TI"][ia] = input->PET[timeStep] * (1 - (folder->VarVecDict["rootZoneStorDef_TI"][ia] / rzMSD));
		//total moisture available to evaporate  (maximum rootzone storage deficit - rootzone storage deficit), for this index value
		double t1 = folder->VarVecDict["rootZoneStorDef_TI"][0];
		double availStorage = rzMSD - folder->VarVecDict["rootZoneStorDef_TI"][ia];
		// if the rootzone ET > is greater then the total moisture available to evaporate  
		if (folder->VarVecDict["rET_TI"][ia] > availStorage) {
			// Set rootzone ET to the difference maximum rootzone storage deficit and rootzone storage deficit - you can't evaporate water that's not there
			// i.e. rzMSD = 0.5 and RzsDeficit = 0 (saturated), actual rootzone ET cannot be greater than 0.5 - the water present
			// i.e. rzMSD = 0.5 and RzsDeficit = 0.35 (not saturated), actual rootzone ET cannot be greater than 0.15 - the water present
			folder->VarVecDict["rET_TI"][ia] = availStorage;      // maxmun allowable ET
		}
		// Reza: For the infiltration trench, I considered the soil layer as zero and then the rzMSD will be zero. 
		// To solve the issue with rET, I set it to zero for the condition of rzMSD = 0
		if (rzMSD == 0.0) { folder->VarVecDict["rET_TI"][ia] = 0.0; }
		// Updating the rootzone storage deficit to reflect ET, by folder and index value
		folder->VarVecDict["rootZoneStorDef_TI"][ia] += folder->VarVecDict["rET_TI"][ia];     // update root zone storage deficit
	}
}

void RootZoneETCalc::calculateRootzoneETTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
	double rzMSD = folder->ParamDict["MSD_m"];
	int ia = input->RepoDict["iA"];
	#define lamda 2257000.0
	// getSoilLE / L + getTreeLET / L + getShortVegLET / L
	double actualEvapoTranspiration = (folder->VarDict["SoilLE"] / lamda) + (folder->VarDict["TreeLET"] / lamda)+ (folder->VarDict["ShortVegLET"] / lamda);
	if (actualEvapoTranspiration > 0.0)
	{
		// Modify rootzone ET to equal = PET * (1 - current rootzone storage deficit / maximum rootzone storage deficit), for this index value
		// the more saturated the rootzone, the lower and closer to 0 the rootzone storage deficit is, and the great the actual ET
		// if rootzone storage deficit = 0, the actualt ET = PET
		folder->VarVecDict["rET_TI"][ia] = actualEvapoTranspiration * (1 - (folder->VarVecDict["rootZoneStorDef_TI"][ia] / rzMSD));
		//total moisture available to evaporate  (maximum rootzone storage deficit - rootzone storage deficit), for this index value
		double t1 = folder->VarVecDict["rootZoneStorDef_TI"][0];
		double availStorage = rzMSD - folder->VarVecDict["rootZoneStorDef_TI"][ia];
		// if the rootzone ET > is greater then the total moisture available to evaporate  
		if (folder->VarVecDict["rET_TI"][ia] > availStorage) {
			// Set rootzone ET to the difference maximum rootzone storage deficit and rootzone storage deficit - you can't evaporate water that's not there
			// i.e. rzMSD = 0.5 and RzsDeficit = 0 (saturated), actual rootzone ET cannot be greater than 0.5 - the water present
			// i.e. rzMSD = 0.5 and RzsDeficit = 0.35 (not saturated), actual rootzone ET cannot be greater than 0.15 - the water present
			folder->VarVecDict["rET_TI"][ia] = availStorage;      // maxmun allowable ET
		}
		// Updating the rootzone storage deficit to reflect ET, by folder and index value
		folder->VarVecDict["rootZoneStorDef_TI"][ia] += folder->VarVecDict["rET_TI"][ia];     // update root zone storage deficit
	}
}

void RootZoneETCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors)
{
	// Resetting the SumET for each timestep/folder - run against V5 to make sure it works the same in semi-distributed mode (11/5/14) - it does ! 11/9/14
	folder->VarDict["sumET"] = 0.0;

	// rzMSD = Maximum root zone storage deficit - set in the param file
	double rzMSD = folder->ParamDict["MSD_m"];

	// This for loop runs for each member of the Topographic Index Distribution - typically 30 values
	for(int ia=0; ia< input->IndexAreaSize; ia++)
	{
		if (timeStep == 0)
		{
			folder->VarVecDict["rET_TI"].push_back(0.0);
		}
		// Setting the rootzone ET equal to 0, for this index value
        folder->VarVecDict["rET_TI"][ia] = 0.0;
		input->RepoDict["iA"] = ia;
		functors["calculateRootzoneET"](input, folder, timeStep);
		// Total ET - for the whole run period? - was (incorrectly) but isn't now due to resetting the value at the beginning of this function
		// Total ET - For this timestep is previous ET value + the amount of ET from this index value * the fraction of the watershed area for this index value
		folder->VarDict["sumET"] += folder->VarVecDict["rET_TI"][ia] * input->IndexArea[ia];              // calculate total ET from the subcatchment
		// In affect this appears to be averaging the ET across the watershed (by Index areas - when in sem-dist mode) while summing it to total ET
	}

	storeValues(input, folder, timeStep);
	
}



void RootZoneETCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{

	folder->VarDict["totalET"] += folder->VarDict["sumET"];
	folder->VarDict["prevSumET"] = folder->VarDict["sumET"];
	//Arpit
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
		folder->VarVecDict["rET"].emplace_back(folder->VarDict["sumET"]);
}
