#ifndef HEATFLUXCAL_H
#define HEATFLUXCAL_H
///////////////////////////////////////////////////////////////
//  HEATFLUXCAL.h - Class to calculate the mesoscale Tb and eb
//            and each folder local temperature and humidity   //
//             ver 1.0                                      //
//                                                           //
//  Language:     2010 Visual studio C++                     //
//  Platform:     DELL                                       //
//  Application:  iTree-Temperature                           //
//  Author:       Yang Yang,  SUNY-ESF & USDA FS             //
//                yyang31@syr.edu                            //
///////////////////////////////////////////////////////////////
/*
Module Operations: 
  ==================
  constructor: HeatFluxCal(InputRepo* inp);  
  This class is to calculate all the heat fluxes, and temperature terms for a folder
  

  Public Interface:
  =================
  HeatFluxCal hfc(*inp);
  

  // there are 4 categories of urban type. This function is to assign the urban variable values based on the urban type.
  hfc.assignUrbanParam(); 

  // based on the urban type, land cover percentages and height of different land cover, calculate the average height the folder canopy
  hfc.CalaveHeight();

  // calculate the fixed terms for a folder at one time step, so they will not be involved in the iterations. Save computational time
  hfc.StateCal(DataFolder *folder, int ts);

  // calculate mesoscale temperatures
  hfc.MesoCal(DataFolder *folder, int ts);

  // calculate the mesoscale temperatures by iteration
  hfc.IterationCalTb(DataFolder *folder, double ts);

  // save calculated heat terms to a folder
  hfc.savefolderValue(DataFolder *folder);

  // calculate local Temperature
  hfc.LocalTCal(DataFolder *folder, DataFolder* refcl, int ts);

  // calculate the shared variables for all folders at one time step, so save some computational time. 
  hfc.VariableForAllfolders(int ts);

  Build Process:
  ==============
  Required files
    - HeatFluxCal.cpp
	
  Build commands 
    - devenv Temperature2014.sln
    
      Maintenance History:
      ====================
      ver 1.0 :  Feb 2014
        - first release
	  ver 2.0 :Aug 2014
	   - did major revision of the functions and variables

*/

#include <iostream>
#include <fstream>
#include <vector>
#include <math.h>
#include <stdio.h>
#include "..\DataFolder.h"
#include "..\Inputs\Inputs.h"
#include "ResistanceCal.h"

#define lamda 2257000.0
#define Cp 1013.0
#define rou 1.205
#define kk 0.40


///////////////////////////////////////////////////////////////
//      HeatCal class declarations                           //
///////////////////////////////////////////////////////////////
//class TemperatureRepo;
class HeatFluxCal
{

	private:
		//TemperatureRepo* repo;
		//double Tb_ref;  // Tb at referecne station
		//double Eb_ref;  //Eb at reference station
		int UrbanCanyon; // urban canyon toggle

		double Tb;  // mesoscale air temperature in K at current calculation
		double Eb;  // mescoscale air absolute humidity
		double Tb_i; // mesoscale air temperature in K at one time step before
		
		double Ta; // canopy air temperature
		double Td;  // canopy dew point temperature
		double windsp; // wind speed
		
	double esatRTa;    // saturated vapor pressure (absolute humidity Kg/m^3) at Ta
    double Rea;  // real vapor pressure (absolute humidity Kg/m^3 at Td)
    double delta;  
	double downLW;  // down long wave radiation
	double upLW;    // up long wave radiation
	double DirSWR;   //direct shortwave radiation
	double DiffSWR;  // diffusive shortwave radiation
	double totalSWR;      // total short wave radiation
	double wateronImp;        // available water on Impervious for evaporate
	double wateronTree;       // available water on Tree for evaporate
	double wateronShort;      // available water on short vegetation for evaporate 
	double wateronSoil;       // available water in the soil for evaporate

	double TreeLAI;  // Tree LAI
	double ShortLAI;  // short vegetation LAI
//	double hwr;  // building height/roadwidth
	double aveHeight;   // canopy average height
		// impervious intermediate heat flux values
		double ImpNR;         // impervious net radiation
		double ImpDeltaQ;     // impervious delta Q (stored Q)
		double ImpC;
		double ImpH;
		double ImpLE;

		double SoilNR;
		double SoilDeltaQ;
		double SoilC;
		double SoilH;
		double SoilLE;

		double TreeNR;
		double TreeDeltaQ;
		double TreeCE;
		double TreeCT;
		double TreeC;
		double TreeH;
		double TreeLEE;
		double TreeLET;
		double TreeLE;

		double ShortNR;
		double ShortDeltaQ;
		double ShortCE;
		double ShortCT;
		double ShortC;
		double ShortH;
		double ShortLEE;
		double ShortLET;
		double ShortLE;

		double WaterNR;
		double WaterDeltaQ;
		double WaterC;
		double WaterH;
		double WaterLE;
		
		double H;
		double LE;


	
		// calculate the state parameters
		// Ta is the canopy air temperature, Td is the canopy dew point temperature

	// soil parameters
	double Soilheight;		// average soil height  rcc 4/22/2020 updating aveHeight based on email w/ Ted et al 4/13/2020
	double Soilalbedo;                                 // soil effective albedo
	double Soilemissivity;                            // soil emissivity
    double SoilMSD;

	double Soila1;
	double Soila2;
	double Soila3;

// urban parameters
    double Urbanalbedo;
	double Urbanheight;
	double Urbanroadwidth;
	double Urbanemissivity;
	double Urbana1;
	double Urbana2;
	double Urbana3;
	double UrbanbuildingP;
	double UrbanDepression;

	// parameters calculated based on the parameters above
	double Urbanhwr;
    double UrbanRVF;
    double UrbanWVF;

	double Urban21albedo;
	double Urban21height;
	double Urban21roadwidth;
	double Urban21emissivity;
	double Urban21depression;
	double Urban21a1;
	double Urban21a2;
	double Urban21a3;
	double Urban21buildingP;

	double Urban22albedo;
	double Urban22height;
	double Urban22roadwidth;
	double Urban22emissivity;
	double Urban22depression;
	double Urban22a1;
	double Urban22a2;
	double Urban22a3;
	double Urban22buildingP;
//	double Urban22Depression;



    double Urban23albedo;
	double Urban23height;
	double Urban23roadwidth;
	double Urban23emissivity;
	double Urban23depression;
	double Urban23a1;
	double Urban23a2;
	double Urban23a3;
	double Urban23buildingP;
//	double Urban23Depression;



	double Urban24albedo;
	double Urban24height;
	double Urban24roadwidth;
	double Urban24emissivity;
	double Urban24depression;
	double Urban24a1;
	double Urban24a2;
	double Urban24a3;
	double Urban24buildingP;
//	double Urban24Depression;



		//tree parameters

	double Treea1;
	double Treea2;
	double Treea3;

	double Treeheight;                         //average tree height
	double Treealbedo;                         //effective albedo
	double Treeemissivity;                     //emissivity
	double TreeSL;                         //canopy maximum water storage depth (mm)

	// short vegetation parameters
	double Shorta1;
	double Shorta2;
	double Shorta3;

	double Shortheight;                         //average tree height
	double Shortalbedo;                         //effective albedo
	double Shortemissivity;                     //emissivity
	double ShortSL;                         //canopy maximum water storage depth (mm)

	double height;        // canopy height. if the folder is urban, then it is the building height, otherwise, it is the tree height or short vegetation height (if tree % =0)

// water parameters

	double Waterheight;		// average water height  rcc 4/22/2020 updating aveHeight based on email w/ Ted et al 4/13/2020
	double Wateralbedo;
	double Wateremissivity;
	double Watera1;
	double Watera2;
	double Watera3;
	ResistanceCal* resist;


    public:
		HeatFluxCal(Inputs* input);
		HeatFluxCal(){};
		~HeatFluxCal(){};
		void assignUrbanParam(int land);
		void CalaveHeight(double treeP, double shortVegP, double soilP, double waterP, double imperP, int land); //  rcc 4/22/2020 modified to fix aveHeight calc as per 4/13/2020 email with Ted et al
		void StateCal(DataFolder *folder, int ts, Inputs *inputs, int folderID);
		void MesoCal(DataFolder *folder, int ts);
		void IterationCalTb(DataFolder *folder, double ts);
		bool IterationCalTbTemp(DataFolder *folder, double ts);
		void savefolderValue(DataFolder *folder, DataFolder* reffolder, int timestep, int indexOfFolder);

		void LocalTCal(DataFolder *folder, DataFolder* refcl, int ts, int iterationCount);
		void VariableForAllfolders(int ts, Inputs *input);
		void assignRefTaTd(Inputs *input, int ts);
		void calDirSWRad(int folderID, Inputs  *input, int timeStep, double &dirSWR);
};


#endif