#include "TestSimulation.h"
#include "Interception/TreeInterceptionCalc.h"
#include "Interception/ShortVegInterceptionCalc.h"
#include "Evaporation/TreeEvaporationCalc.h"
#include "Evaporation/ShortVegEvaporationCalc.h"
#include "ThroughFlow/PerviousAreaThroughFlowCalc.h"
#include "SnowMelt/SnowMeltOpenAreaCalc.h"
#include "SnowMelt/SnowMeltUnderShortVegCalc.h"
#include "SnowMelt/SnowMeltUnderTreeCalc.h"
#include "WaterOnImpArea/WaterOnImperviousAreaCalc.h"
#include "ImperviousDepressionStorage/ImperviousDepressionStorageCalc.h"
#include "WaterOnPerArea/WaterOnPerviousAreaCalc.h"
#include "PerviousDepressionStorage/PerviousDepressionStorageCalc.h"
#include "InfiltrationPowerDecay/InfiltrationPowDecayCalc.h"
#include "InfiltrationExpDecay/InfiltrationExpDecayCalc.h"
#include "SemiAveSMD/SemiAveSMDCalc.h"
#include "StoreTimeStepData/StoreTimeStepTotalsCalc.h"
#include "RootZone/RootZoneETCalc.h"
#include "RootZone/RootZoneFlowCalc.h"
#include "UnSatZone/UnsatZonePerviousFlowCalc.h"
#include "UnSatZone/UnsatZoneFlowCalc.h"
#include "UnSatZone/UnsatZoneSoilMoistureDeficitExpDecayCalc.h"
#include "UnSatZone/UnsatZoneSoilMoistureDeficitPowDecay.h"
#include "UnSatZone/UnsatZoneStorageCalc.h"
#include "SatZone/SaturatedBaseFlowExpDecayCalc.h"
#include "SatZone/SaturatedBaseFlowPowDecay.h"
#include "Flow/FlowRouteCalc.h"
#include "Flow/FlowSumCalc.h"
#include "Outputs/StatisticalOutputWriter.h"
#include "Outputs/SpatialOutputWriter.h"
#include "Outputs/GIOutputWriter.h"
#include "Temperature/HeatFluxCal.h"
#include "Temperature/TemperatureOutputWriter.h"
#include "Temperature/BlockGroupHandler.h"
#include "Temperature/ICTCInterpolater.h"
#include "GI/GI_Outlets.h"
#include <algorithm>
#include <limits>
#include <cstddef>
#include <string>
#include <initializer_list>
#include "./Solar/SolarCalculation.h"
#include "./Solar/StreamTemperature.h"

void TestSimulation::Go(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	std::cout << "Model: " << input->SimulationStringParams["Model"] << std::endl;
	std::cout << "Infiltration: " << input->SimulationStringParams["Infiltration"] << std::endl;
	std::cout << "Routing: " << input->SimulationStringParams["Routing"] << std::endl;
	std::cout << "Running Simulation..." << std::endl;
	LogMessage::log("Model: " + input->SimulationStringParams["Model"]);
	LogMessage::log("Infiltration: " + input->SimulationStringParams["Infiltration"]);
	LogMessage::log("Routing: " + input->SimulationStringParams["Routing"]);
	LogMessage::log("Running Simulation...");
	if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
		RunStatisticalModel(organizer, input, inputDirectory);
	}
	else if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		RunTemperatureSpatialModel(organizer, input, inputDirectory);
	}
	if (input->SimulationStringParams["Model"] == "CoolRiver")
	{
		RunCoolRiverModel(organizer, input, inputDirectory);
	}
}

void TestSimulation::RunTemperatureSpatialModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	BlockGroupHandler handleBlockGroups(input);
	std::vector<double> waterOnValues;
	HeatFluxCal heatC(input);
	bool needDailyBGFiles = (input->SimulationNumericalParams["TotalTimeSteps"] >= 24);
	bool needHourlyBGFiles = (input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "true" || input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "True");
	TemperatureOutputWriter::generateBlockFiles(handleBlockGroups.BlockGroupIDToItsFolder, input->SimulationStringParams["OutputDirectory"], needHourlyBGFiles, needDailyBGFiles);
	TemperatureOutputWriter::generateRowColFiles(organizer, input);
	int refLocation = input->TemperatureCalculationParams["RefWeatherLocation"];

	DataFolder *refLocationFolder = organizer->DataDrawers[refLocation][0];
	double waterOnValuesIndex = 0;
	for (int timeStep = 0; timeStep < input->SimulationNumericalParams["TotalTimeSteps"]; ++timeStep)
	{
		if (timeStep > 0)
		{
			heatC.VariableForAllfolders(timeStep, input);
			heatC.assignRefTaTd(input, timeStep);
			heatC.assignUrbanParam(input->NLCDLandCoverData[refLocation]);
			heatC.CalaveHeight(refLocationFolder->ParamDict["TreeP"], refLocationFolder->ParamDict["ShortVegP"], refLocationFolder->ParamDict["SoilP"], refLocationFolder->ParamDict["WaterP"], refLocationFolder->ParamDict["ImperP"], input->NLCDLandCoverData[refLocation]); // rcc 4/22/2020 modified to fix aveHeight calc as per 4/13/2020 email with Ted et al
			heatC.StateCal(refLocationFolder, timeStep, input, refLocation); // calculate meso-scale temperature and humidity
			heatC.IterationCalTb(refLocationFolder, timeStep);
			heatC.savefolderValue(refLocationFolder, refLocationFolder, timeStep, refLocation);
		}

		for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
		{
			for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
			{
				if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "NODATA")
				{
					//std::cout<<"continue";
				}
				else
				{
					if (timeStep > 0 && organizerLocationIndex != refLocation)
					{
						heatC.assignRefTaTd(input, timeStep);
						heatC.assignUrbanParam(input->NLCDLandCoverData[organizerLocationIndex]);
						heatC.CalaveHeight(organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["TreeP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["ShortVegP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["SoilP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["WaterP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["ImperP"], input->NLCDLandCoverData[organizerLocationIndex]); // rcc 4 / 22 / 2020 modified to fix aveHeight calc as per 4 / 13 / 2020 email with Ted et al
						heatC.StateCal(organizer->DataDrawers[organizerLocationIndex][folderIndex], timeStep, input, organizerLocationIndex); // calculate meso-scale temperature and humidity
						heatC.LocalTCal(organizer->DataDrawers[organizerLocationIndex][folderIndex], refLocationFolder, timeStep, input->TemperatureCalculationParams["IterationCount"]);
						heatC.savefolderValue(organizer->DataDrawers[organizerLocationIndex][folderIndex], refLocationFolder, timeStep, organizerLocationIndex);
					}
					runCalculations(organizer->DataDrawers[organizerLocationIndex][folderIndex], input, timeStep, organizer->DataDrawers[organizerLocationIndex]);
					FlowSumCalc::addfolderFlowValues(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], timeStep);
					TemperatureOutputWriter::rowColFileBatchProcessor(timeStep, organizerLocationIndex, organizer->DataDrawers[organizerLocationIndex][folderIndex], input->SimulationStringParams["OutputDirectory"], input);
					if (timeStep > 0)
						handleBlockGroups.AddToBlockGroup(organizerLocationIndex, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
					if (timeStep == 1 && input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
						TemperatureOutputWriter::outputLandCoverDetails(organizerLocationIndex, input->SimulationStringParams["OutputDirectory"], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				}
			}
		}
		FlowRouteCalc::calculateStatistical(input, timeStep);
		if (timeStep > 0)
			handleBlockGroups.TimeStepProcessing(input, timeStep);
		TemperatureOutputWriter::timeStepResultsFileProcessor(timeStep, organizer, input->SimulationStringParams["OutputDirectory"], input);
	}
	if (input->TemperatureExecutionParams["LCScalingOutput"] == "Yes")
	{
		ICTCInterpolator interpolator;
		std::cout << "===" << std::endl;
		std::cout << "Running LCScalingOutput interpolation" << std::endl;
		LogMessage::log("Running LCScalingOutput interpolation");
		std::cout << "===" << std::endl;
		interpolator.interpolateData(input);
	}
	std::cout << "===" << std::endl;
	std::cout << "Writing output to " << std::endl;
	LogMessage::log("Writing output to " + input->SimulationStringParams["OutputDirectory"]);
	std::cout << input->SimulationStringParams["OutputDirectory"] << std::endl;
	std::cout << "===" << std::endl;
}

void TestSimulation::RunCoolRiverModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	std::vector<double> solarAzimuth;
	std::vector<double> solarAltitude;

	SolarCalculation *sc = new SolarCalculation(input);
	int timeSteps = input->WindSp_mps.size();
	//check that SolarRadiation.dat & Weather.dat have enough rows for totTime param in config
	if (input->WindSp_mps.size() != timeSteps)
	{
		std::ofstream logfile("log.txt", ios::app);
		logfile << "Weather.dat data entries should equal totTime param in config";
		logfile << "\n";
		logfile.close();
		return;
	}
	int x = input->DirectShortWaveRadiation.size();
	if (input->DirectShortWaveRadiation.size() != timeSteps)
	{
		std::ofstream logfile("log.txt", ios::app);
		logfile << "SolarRadiation.dat data entries/rows should be equal totTime param in config";
		logfile << "\n";
		logfile.close();
		return;
	}
	for (int ts = 0; ts < timeSteps; ++ts)
	{
		sc->SolarHour(ts);
		solarAltitude.push_back(sc->getSolarAltitude());
		solarAzimuth.push_back(sc->getSolarAzimuth());
	}
	StreamTemperature *st = new StreamTemperature(input, sc); // Added by Reza Abdi (Jan. 17)
	st->avgTemperature();
	int TotalTimeMod = input->CoolRiverVariables["CR_totTime"];
	int TotalDistMod = input->CoolRiverVariables["CR_totDist"];
	int dx_Uns = input->CoolRiverVariables["CR_unsteady_dx"];
	int dx_HR = input->CoolRiverVariables["CR_interval_dx"];
	double dtime_IntervalHR = input->CoolRiverVariables["CR_unsteady_dt"]; // in second
	int dt_HR = input->CoolRiverVariables["CR_interval_dt"];
	int calcMethod = input->CoolRiverVariables["CR_calcMethod"];
	//int TotalTimeModHR = TotalTimeMod * 60 * (60.0 / dt_HR);
	int TotalTimeModHR = TotalTimeMod;
	int TotalDist_Uns;
	if (calcMethod == 3) {
		TotalDist_Uns = (TotalDistMod / dx_Uns) + 1;
	}
	else {
		TotalDist_Uns = (TotalDistMod / dx_Uns);
	}
	int TotalDist_Uns_HR = (TotalDistMod / dx_HR);

	// Crank-Nicolson method
	int TotalDistMod_CN = 1;
	int TotalTimeMod_CN = 1;
	if (calcMethod == 1)
	{
		TotalDistMod_CN = TotalDistMod;
		TotalTimeMod_CN = TotalTimeMod;
	}
	std::vector<vector<double>> modeledT_x0(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> skyViewFactorE(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> skyViewFactorW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> atmLW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> landCoverLW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> upToAirLW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> LW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> latent(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> sensible(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> sediment(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> thermalExchange(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> SW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> dirSW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	std::vector<vector<double>> diffSW(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN));
	//std::vector<vector<double>> t_minBased1(TotalDistMod_CN, std::vector<double>(TotalTimeMod_CN * 60));

	if (calcMethod == 1)
	{
		// For Steady State
		modeledT_x0 = st->getModT();
		dirSW = st->getSWDir_2D();
		diffSW = st->getSWDiff_2D();
		atmLW = st->getAtmLW();
		landCoverLW = st->getLcLW();
		upToAirLW = st->getBackLW();
		LW = st->getLW();
		latent = st->getLatent();
		sensible = st->getSensible();
		sediment = st->getBed();
		thermalExchange = st->getHeatFlux();
		SW = st->getSW();
		//t_minBased1 = st->getT_MinBased();
	}

	int TotalDistMod_FD = 1;
	int TotalTimeUns = 1;
	if (calcMethod == 2)
	{
		TotalDistMod_FD = TotalDistMod;
		TotalTimeUns = TotalTimeMod * 60;
	}

	std::vector<vector<double>> Flux_Total_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_Conduction_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_Evaporation_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_Convection_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> FLux_AtmLW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_LCLW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_BackLW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_LW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_DirSW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_DiffSW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Flux_SW_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> T_Uns_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Depth_Uns_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Pw_unsteady_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> width_UnsTotal_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> depth_AvgUns_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Hyporheic_Exchange_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));
	std::vector<vector<double>> Rh_unsteady_copy(TotalDistMod_FD, std::vector<double>(TotalTimeUns));

	std::vector<vector<double>> temp_Flux_Total(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_Conduction(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_Evaporation(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_Convection(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_FLux_AtmLW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_LCLW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_BackLW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_LW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_DirSW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_DiffSW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Flux_SW(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_T_Uns(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_depth_Uns(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Pw_unsteady(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_width_UnsTotal(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_depth_AvgUns(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Hyporheic_Exchange(TotalDist_Uns, std::vector<double>(TotalTimeMod));
	std::vector<vector<double>> temp_Rh_unsteady(TotalDist_Uns, std::vector<double>(TotalTimeMod));

	//std::vector<double> temp_Hyporheic_Exchange(TotalDist_Uns);
	//std::vector<double> temp_Q_GW(TotalDist_Uns);

	if (calcMethod == 2)
	{
		// For Unsteady state
		Flux_Total_copy = st->get_TotalFlux();
		Flux_Conduction_copy = st->get_Conduction();
		Flux_Evaporation_copy = st->get_Evaporation();
		Flux_Convection_copy = st->get_Convection();
		FLux_AtmLW_copy = st->get_AtmLW();
		Flux_LCLW_copy = st->get_LCLW();
		Flux_BackLW_copy = st->get_BackLW();
		Flux_LW_copy = st->get_Flux_LW();
		Flux_DirSW_copy = st->get_Flux_DirSW();
		Flux_DiffSW_copy = st->get_Flux_DiffSW();
		Flux_SW_copy = st->get_Flux_SW();
		T_Uns_copy = st->get_T_Uns();
		Depth_Uns_copy = st->get_depth_Uns();

		temp_Flux_Total = st->get_hour_TotalFlux();
		temp_Flux_Conduction = st->get_hour_Conduction();
		temp_Flux_Evaporation = st->get_hour_Evaporation();
		temp_Flux_Convection = st->get_hour_Convection();
		temp_FLux_AtmLW = st->get_hour_AtmLW();
		temp_Flux_LCLW = st->get_hour_LCLW();
		temp_Flux_BackLW = st->get_hour_BackLW();
		temp_Flux_LW = st->get_hour_Flux_LW();
		temp_Flux_DirSW = st->get_hour_Flux_DirSW();
		temp_Flux_DiffSW = st->get_hour_Flux_DiffSW();
		temp_Flux_SW = st->get_hour_Flux_SW();
		temp_T_Uns = st->get_hour_T_Uns();
		temp_depth_Uns = st->get_hour_depth_Uns();

		//temp_Hyporheic_Exchange = st->getHypEx();
		//temp_Q_GW = st->getQ_GW();
	}

	// The vectors required for the HEC_RAS related calculation method (Jan. 18)
	// start

	int TotalDistMod_HR = 1;
	int TotalTime_HR = 1;
	if (calcMethod == 3)
	{
		TotalDistMod_FD = TotalDistMod;
		TotalTime_HR = (TotalTimeModHR - 1) * 60;
	}

	std::vector<vector<double>> temp_Flux_Evaporation_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_Conduction_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_Convection_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_FLux_AtmLW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_LCLW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_BackLW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_LW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_DirSW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_DiffSW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_SW_HR(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_Total_HRdist(TotalDistMod_FD, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Temp_HRdist(TotalDistMod_FD, std::vector<double>(TotalTime_HR));

	std::vector<vector<double>> temp_Flux_Evaporation_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_Conduction_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_Convection_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_FLux_AtmLW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_LCLW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_BackLW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_LW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_DirSW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_DiffSW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Flux_SW_Nodes(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_FluxTotal_HR(TotalDist_Uns, std::vector<double>(TotalTime_HR));
	std::vector<vector<double>> temp_Temp1_HRsize2(TotalDist_Uns, std::vector<double>(TotalTime_HR));

	// For Debugging purpose-start
	//std::vector<vector<double>> temp_tempNodesDebugg(12, std::vector<double>(96));
	//temp_tempNodesDebugg = st->get_tempNodesDebugg();
	//std::vector<double> tempHyporheic_Exchange_HR(100);
	//tempHyporheic_Exchange_HR = st->getHypEx_HR();
	// For Debugging purpose-end

	if (calcMethod == 3)
	{
		temp_Flux_Evaporation_HR = st->get_EvapHRdist();
		temp_Flux_Conduction_HR = st->get_CondHRdist();
		temp_Flux_Convection_HR = st->get_ConvHRdist();
		temp_FLux_AtmLW_HR = st->get_AtmLWHRdist();
		temp_Flux_LCLW_HR = st->get_LCLWHRdist();
		temp_Flux_BackLW_HR = st->get_BackLWHRdist();
		temp_Flux_LW_HR = st->get_LWHRdist();
		temp_Flux_DirSW_HR = st->get_DirSWHRdist();
		temp_Flux_DiffSW_HR = st->get_DiffSWHRdist();
		temp_Flux_SW_HR = st->get_SWHRdist();
		temp_Flux_Total_HRdist = st->get_TotalFluxHRdist();
		temp_Temp_HRdist = st->get_TempHRdist();

		temp_Flux_Evaporation_Nodes = st->get_EvapHRnodes();
		temp_Flux_Conduction_Nodes = st->get_CondHRnodes();
		temp_Flux_Convection_Nodes = st->get_ConvHRnodes();
		temp_FLux_AtmLW_Nodes = st->get_get_AtmLWHRnodes();
		temp_Flux_LCLW_Nodes = st->get_LCLWHRnodes();
		temp_Flux_BackLW_Nodes = st->get_BackLWHRnodes();
		temp_Flux_LW_Nodes = st->get_LWHRnodes();
		temp_Flux_DirSW_Nodes = st->get_DirSWHRnodes();
		temp_Flux_DiffSW_Nodes = st->get_DiffSWHRnodes();
		temp_Flux_SW_Nodes = st->get_SWHRnodes();
		temp_FluxTotal_HR = st->get_TotalFluxHRnodes();
		temp_Temp1_HRsize2 = st->get_TempHRnodes();
	}

	switch (calcMethod)
	{
	case 1:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {

		// Writing the results for the steady state condition
        //Added River Station and time Step Strings as headers for output files
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", st->T, "Hourly_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", dirSW, "Hourly_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", diffSW, "Hourly_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", atmLW, "Hourly_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", landCoverLW, "Hourly_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", upToAirLW, "Hourly_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", LW, "Hourly_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", latent, "Hourly_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", sensible, "Hourly_SensibleE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", st->heat_flux_2D, "Hourly_TotalHeatFlux_J_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", SW, "Hourly_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		twoDwriter(TotalDistMod, TotalTimeMod, "River Station", "Time step", sediment, "Hourly_SedimentConduction_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		}
		else {
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->t_minBased, "Minutely_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->heat_flux_2D_min, "Minutely_TotalHeatFlux_J_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->SWDir_2D_min, "Minutely_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->SWDiff_2D_min, "Minutely_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->SW_2D_min, "Minutely_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->LW_2D_min, "Minutely_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->atmLW_2D_min, "Minutely_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->LCLW_2D_min, "Minutely_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->backLW_2D_min, "Minutely_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->Latent_2D_min, "Minutely_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->Sensible_2D_min, "Minutely_SensibleE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns, (TotalTimeMod_CN-1) * 60, "River Station", "Time step", st->Bed_2D_min, "Minutely_SedimentConduction_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		}

		break;
	case 3:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
		
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->DeltaTotal_HR, "Hourly_DeltaTotalHR", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->DepthAvgALL_HR, "Hourly_RiverAvgDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->AreaALL_HR, "Hourly_RiverXSArea_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->VelocityALL_HR, "Hourly_RiverVelocity_m_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->WidthALL_HR, "Hourly_RiverWidth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->RetentionT_HR, "Hourly_RiverRetentionTime_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->Pw_ALL_HR, "Hourly_RiverWettedPerimeter_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->WSElev_ALL_HR, "Hourly_RiverWaterSurfElev_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->Q_Str_HR, "Hourly_RiverQDischarge_m3_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->HyporheicExchange_HR, "Hourly_HyporheicExchange_m3_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", st->Dispersion_HR, "Hourly_LongDispersion_m2_per_s", input->SimulationStringParams["OutputDirectory"]);
			}
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Temp1_HRsize2, "Hourly_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_FluxTotal_HR, "Hourly_TotalHeatFlux_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_SW_Nodes, "Hourly_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_DirSW_Nodes, "Hourly_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_DiffSW_Nodes, "Hourly_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_LW_Nodes, "Hourly_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_LCLW_Nodes, "Hourly_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_BackLW_Nodes, "Hourly_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_FLux_AtmLW_Nodes, "Hourly_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_Convection_Nodes, "Hourly_SensibleE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_Conduction_Nodes, "Hourly_SedimentConduction_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTimeModHR, "River Station", "Time step", temp_Flux_Evaporation_Nodes, "Hourly_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->delta_2D_min, "Minutely_DeltaTotalHR", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->depthAvgALL_2D_min, "Minutely_RiverAvgDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->AreaALL_2D_min, "Minutely_RiverXSArea_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->VelocityALL_2D_min, "Minutely_RiverVelocity_m_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->WidthALL_2D_min, "Minutely_RiverWidth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->RetentionT_2D_min, "Minutely_RiverRetentionTime_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Pw_ALL_2D_min, "Minutely_RiverWettedPerimeter_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->WSElev_ALL_2D_min, "Minutely_RiverWaterSurfElev_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Q_Str_2D_min, "Minutely_RiverQDischarge_m3_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->HyporheicExchange_2D_min, "Minutely_HyporheicExchange_m3_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Dispersion_2D_min, "Minutely_LongDispersion_m2_per_s", input->SimulationStringParams["OutputDirectory"]);
			}
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->t_minBased, "Minutely_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->heat_flux_2D_min, "Minutely_TotalHeatFlux_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->SWDir_2D_min, "Minutely_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->SWDiff_2D_min, "Minutely_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->SW_2D_min, "Minutely_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->LW_2D_min, "Minutely_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->atmLW_2D_min, "Minutely_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->LCLW_2D_min, "Minutely_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->backLW_2D_min, "Minutely_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Latent_2D_min, "Minutely_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Sensible_2D_min, "Minutely_SensibleE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			twoDwriter(TotalDist_Uns_HR, TotalTime_HR, "River Station", "Time step", st->Bed_2D_min, "Minutely_SedimentConduction_W_per_m2", input->SimulationStringParams["OutputDirectory"]);

		}
		break;
	case 2:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_depth_Uns, "Hourly_WaterDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_Pw_unsteady, "Hourly_WettedPerimeter_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_width_UnsTotal, "Hourly_WaterWidth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_depth_AvgUns, "Hourly_WaterAvgDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_Hyporheic_Exchange, "Hourly_HyporheicExchange", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_Rh_unsteady, "Hourly_HydraulicRadius_m", input->SimulationStringParams["OutputDirectory"]);
			}
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_Flux_Total, "Hourly_TotalHeatFlux_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", st->hour_T_Uns, "Hourly_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_Conduction, "Hourly_SedimentConduction_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_Evaporation, "Hourly_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_Convection, "Hourly_Sensible_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_FLux_AtmLW, "Hourly_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_LCLW, "Hourly_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_BackLW, "Hourly_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_LW, "Hourly_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_DirSW, "Hourly_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_DiffSW, "Hourly_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, TotalTimeMod, "River Station", "Time step", temp_Flux_SW, "Hourly_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", Depth_Uns_copy, "Minutely_RiverDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Pw_unsteady_2D_min, "Minutely_RiverWettedPerimeter_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->width_UnsTotal_2D_min, "Minutely_RiverWidth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->depth_AvgUns_2D_min, "Minutely_RiverAvgDepth_m", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Hyporheic_Exchange_2D_min, "Minutely_HyporheicExchange_m3_per_s", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Rh_unsteady_2D_min, "Minutely_HydraulicRadius_m", input->SimulationStringParams["OutputDirectory"]);
				}
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->t_minBased, "Minutely_Temperature_C_deg", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->heat_flux_2D_min, "Minutely_TotalHeatFlux_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Bed_2D_min, "Minutely_SedimentConduction_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Latent_2D_min, "Minutely_LatentE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->Sensible_2D_min, "Minutely_SensibleE_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->atmLW_2D_min, "Minutely_AtmLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->LCLW_2D_min, "Minutely_LandcoverLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->backLW_2D_min, "Minutely_WaterBack2SkyLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->LW_2D_min, "Minutely_TotalLW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->SWDir_2D_min, "Minutely_DirectSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->SWDiff_2D_min, "Minutely_DiffuseSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
				twoDwriter(TotalDist_Uns, (TotalTimeMod - 1) * 60, "River Station", "Time step", st->SW_2D_min, "Minutely_TotalSW_W_per_m2", input->SimulationStringParams["OutputDirectory"]);
			}
		
		break;

	default:
		std::ofstream logfile3("log.txt", ios::app);
		logfile3 << "Wrong modeling methodology code! Check the methodology code and try again.";
		logfile3 << "\n";
		logfile3.close();
		cout << "Wrong modeling methodology code! Check the methodology code and try again." << endl;
		break;
	}

	delete sc;
	std::ofstream logfile4("log.txt", ios::app);
	logfile4 << "End of Simulation...";
	logfile4 << "\n";
	logfile4.close();
	std::cout << "End of Simulation..." << std::endl;
	//std::cout << std::endl << "Hit any key and then press enter to close." << std::endl;
	//std::cin.get();
}
void TestSimulation::twoDwriter(int sizeX, int sizeY, string a, string b, vector<vector<double>> &vector, string title, string OutputDir)
{

	title = OutputDir + "\\2D_" + title + ".csv";
	ofstream outfileC(title);

	if (!outfileC.good())
	{
		std::ofstream logfile("log.txt", ios::app);
		logfile << "Cannot write the output file: " + title;
		logfile << "\n";
		logfile.close();
		std::cout << "Cannot write the output file" << std::endl;
		return;
	}
	vector.resize(sizeX);
	//adding row and column headers for the output files.
	outfileC << ",";
	//k is representing time step to handle in separate loop
	for (int k = 0; k < sizeY; k++)
	{
		//taking string values as an input to column headers
		outfileC << b
			<< "_" << k << ",";
	}
	outfileC << '\n';

	for (int i = 0; i < sizeX; i++)
	{
		//taking string values as an input to row headers
		outfileC << a << "_" << i
			<< ",";
		vector[i].resize(sizeY);
		for (int j = 0; j < sizeY; j++)
		{
			//taking the 2D vectors as output i = river station, j = time step
			outfileC << vector[i][j] << ",";
		}
		//next line
		outfileC << '\n';
	}
	std::ofstream logfile2("log.txt", ios::app);
	logfile2 << "Finished Writing Output file: " + title;
	logfile2 << "\n";
	logfile2.close();
}

void TestSimulation::writeExtendedOutput(DataOrganizer *organizer, Inputs *input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];

	for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
		{
			// New extended output files for water balancing
			if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "BulkArea")
			{
				GIOutputWriter::writeVegCanopyFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeVegCanopyFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnPerAreaFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnPerAreaFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnImpAreaFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnImpAreaFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeEvap_ET_Infil_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeEvap_ET_Infil_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex],organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "BioRetention")
			{
				// BioRetention
				GIOutputWriter::writeWaterBalance_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RainGarden")
			{
				// Rain Garden
				GIOutputWriter::writeWaterBalance_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "InfilTrench")
			{
				// InfilTrench
				GIOutputWriter::writeWaterBalance_InfilTrench(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "Swale")
			{
				// Swale
				GIOutputWriter::writeWaterBalance_Swale(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "PermeablePavement")
			{
				// PermeablePavement
				GIOutputWriter::writeWaterBalance_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "GreenRoof")
			{
				//Li:green roof
				GIOutputWriter::writeWaterBalance_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RoofDiscon")
			{
				//Li:rooftop Disconnection
				GIOutputWriter::writeWaterBalance_RoofDiscon(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RainBarrel")
			{
				//Li: Rain Barrel
				GIOutputWriter::writeWaterBalance_RainBarrel(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			//	GIOutputWriter::writeVegCanopyFile_VWB_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeVegCanopyFile_FWB_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_VWB_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_FWB_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
			//else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "BioRetention")
			//{
			//	// BioRetention
			//	GIOutputWriter::writeVegCanopyFile_VWB_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeVegCanopyFile_FWB_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_VWB_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_FWB_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
			//else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "GreenRoof")
			//{
			//	// Green Roof
			//	GIOutputWriter::writeVegCanopyFile_VWB_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeVegCanopyFile_FWB_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_VWB_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_FWB_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
			//else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "PermeablePavement")
			//{
			//	// Porous Pavement  - Need to modify the below functions if needed
			//	GIOutputWriter::writeVegCanopyFile_VWB_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeVegCanopyFile_FWB_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_VWB_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_FWB_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
			//else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RainBarrel")
			//{
			//	// Rain Barrel - Need to modify the below functions if needed
			//	GIOutputWriter::writeFlow_VWB_RainBarrel(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeFlow_FWB_RainBarrel(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
			//else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "TreePit")
			//{
			//	// Tree Pit - Should be working
			//	GIOutputWriter::writeVegCanopyFile_VWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeVegCanopyFile_FWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_VWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeEvap_ET_Infil_FWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeWaterInRootZone_VWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	GIOutputWriter::writeWaterInRootZone_FWB_TreePit(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			//	StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//	StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			//}
		}
	}
}

void TestSimulation::writeNoExtendedOutput(DataOrganizer *organizer, Inputs *input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];

	for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
		{
			StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex],organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
		}
	}
}

void TestSimulation::writeStatisticalOutput(DataOrganizer *organizer, Inputs *input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];
	//Reza: I changed this part to make multiple similar bulk areas output files available but for some issues in Unified Hudro I think the idea doesn't work properly yet!
	//StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[0][0]);
	//StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[0][0]);
	//StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[0][0]);
	StatisticalOutputWriter::writeLAISeries(input);
	StatisticalOutputWriter::writePollutionFile(inputDirectory, input);
	if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
	{
		StatisticalOutputWriter::writeCaliQobs(input, inputDirectory);
		StatisticalOutputWriter::writeCaliTotalQQ(input);
	}
	if (input->SimulationStringParams["ExtendedOutputsTimeStep"] != "NoExtendedOutputs")
	{
		writeExtendedOutput(organizer, input);
	}
	else
	{
		writeNoExtendedOutput(organizer, input);
	}
}

void TestSimulation::RunStatisticalModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	// Reza
	input->RepoDict["DataFolderNm"] = organizer->DataDrawers[0].size();

	for (int timeStep = 0; timeStep < input->SimulationNumericalParams["TotalTimeSteps"]; ++timeStep)
	{
		//std::cout << timeStep << std::endl;
		for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
		{
			for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
			{
				runProcesses(organizer->DataDrawers[organizerLocationIndex][folderIndex], input, timeStep, organizer->DataDrawers[organizerLocationIndex]);
			}
			FlowSumCalc::calculateStatistical(input, organizer->DataDrawers[organizerLocationIndex], timeStep);
		}
		FlowRouteCalc::calculateStatistical(input, timeStep);
		StoreTimeStepTotalsCalc::calculate(input, timeStep, organizer->DataDrawers);
	}
	std::cout << "===" << std::endl;
	std::cout << "Writing output to " << std::endl;
	std::cout << input->SimulationStringParams["OutputDirectory"] << std::endl;
	std::cout << "===" << std::endl;
	LogMessage::log("Writing output to " + input->SimulationStringParams["OutputDirectory"]);
	writeStatisticalOutput(organizer, input);
}

void TestSimulation::runCalculations(DataFolder *folder, Inputs *input, int timeStep, std::vector<DataFolder *> &drawer)
{
	std::map<std::string, void (*)(Inputs * input, DataFolder * folder, int timeStep)> functors;
	if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationStatistical);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical);
		functors.emplace("calculatePerDepEvap", PerviousDepressionStorageCalc::calculatePerDepEvapStatistical);
		functors.emplace("calculateImpEvap", ImperviousDepressionStorageCalc::calculateImpEvapStatistical);
		functors.emplace("calculateRootzoneET", RootZoneETCalc::calculateRootzoneETStatistical);
	}
	if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationTemperature);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationTemperature);
		functors.emplace("calculatePerDepEvap", PerviousDepressionStorageCalc::calculatePerDepEvapTemperature);
		functors.emplace("calculateImpEvap", ImperviousDepressionStorageCalc::calculateImpEvapTemperature);
		functors.emplace("calculateRootzoneET", RootZoneETCalc::calculateRootzoneETTemperature);
	}
	if (input->SimulationStringParams["Infiltration"] == "PowerDecay")
	{
		functors.emplace("calculateInfiltration", SoilInfiltrationPowDecayCalc::calculate);
		functors.emplace("calculateUnsatZoneSoilMoistureDeficit", UnsatZoneSoilMoistureDeficitPowDecayCalc::calculate);
		functors.emplace("calculateSaturatedBaseFlow", SaturatedBaseFlowPowDecayCalc::calculate);
	}
	if (input->SimulationStringParams["Infiltration"] == "ExponentialDecay")
	{
		functors.emplace("calculateInfiltration", SoilInfiltrationExpDecayCalc::calculate);
		functors.emplace("calculateUnsatZoneSoilMoistureDeficit", UnsatZoneSoilMoistureDeficitExpDecayCalc::calculate);
		functors.emplace("calculateSaturatedBaseFlow", SaturatedBaseFlowExpDecayCalc::calculate);
	}

	// GI related, Reza: If we want to have no base flow for the GIs we can set it to zero in the config file.
	if (folder->ParamStringDict["Type"] == "BulkArea")
	{
		folder->ParamDict["BF_flag"] = 1.0;
	}

	if (folder->ParamStringDict["Type"] == "BulkArea")
	{
		// for the bulk area
		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		PerviousAreaThroughFlowCalc::calculate(input, folder, timeStep);
		SnowMeltOpenAreaCalc::calculate(input, folder, timeStep);
		SnowMeltUnderTreeCalc::calculate(input, folder, timeStep);
		SnowMeltUnderShortVegCalc::calculate(input, folder, timeStep);
		WaterOnImperviousAreaCalc::calculate(input, folder, timeStep);
		ImperviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		WaterOnPerviousAreaCalc::calculate(input, folder, timeStep, drawer);
		PerviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		SemiAveSMDCalc::calculate(input, folder, timeStep);
		functors["calculateInfiltration"](input, folder, timeStep);
		RootZoneFlowCalc::calculate(input, folder, timeStep);
		RootZoneETCalc::calculate(input, folder, timeStep, functors);
		functors["calculateUnsatZoneSoilMoistureDeficit"](input, folder, timeStep);
		UnsatZoneStorageCalc::calculate(input, folder, timeStep);
		UnsatZoneFlowCalc::calculate(input, folder, timeStep);
		UnsatZonePerviousFlowCalc::calculate(input, folder, timeStep);
		functors["calculateSaturatedBaseFlow"](input, folder, timeStep);
	}

	// GI specific function
	if (folder->ParamStringDict["Type"] != "BulkArea")
	{
		// Reza, NEW, initializing

		GI_Outlets::giproc_initWaterBalance(folder);

		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		PerviousAreaThroughFlowCalc::calculate(input, folder, timeStep);
		SnowMeltOpenAreaCalc::calculate(input, folder, timeStep);
		SnowMeltUnderTreeCalc::calculate(input, folder, timeStep);
		SnowMeltUnderShortVegCalc::calculate(input, folder, timeStep);
		WaterOnImperviousAreaCalc::calculate(input, folder, timeStep);
		ImperviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		WaterOnPerviousAreaCalc::calculate(input, folder, timeStep, drawer);
		PerviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);

		GI_Outlets::getGI_Outflow(input, folder, timeStep);

		SemiAveSMDCalc::calculate(input, folder, timeStep);
		functors["calculateInfiltration"](input, folder, timeStep);
		RootZoneFlowCalc::calculate(input, folder, timeStep);
		RootZoneETCalc::calculate(input, folder, timeStep, functors);
		functors["calculateUnsatZoneSoilMoistureDeficit"](input, folder, timeStep);
		UnsatZoneStorageCalc::calculate(input, folder, timeStep);
		UnsatZoneFlowCalc::calculate(input, folder, timeStep);
		UnsatZonePerviousFlowCalc::calculate(input, folder, timeStep);

		// Reza, NEW
		GI_Outlets::gi_getRunoff(input, folder, timeStep);
	}
}

void TestSimulation::runProcesses(DataFolder *folder, Inputs *input, int timeStep, std::vector<DataFolder *> &stafolders)
{

	if ((folder->ParamStringDict["Type"] == "BulkArea") || (folder->ParamStringDict["Type"] == "RainGarden") ||
		(folder->ParamStringDict["Type"] == "BioRetention") || (folder->ParamStringDict["Type"] == "GreenRoof") ||
		(folder->ParamStringDict["Type"] == "RainBarrel") || (folder->ParamStringDict["Type"] == "PermeablePavement") ||
		(folder->ParamStringDict["Type"] == "Swale") || (folder->ParamStringDict["Type"] == "InfilTrench") ||
		(folder->ParamStringDict["Type"] == "RoofDiscon"))
	{
		runCalculations(folder, input, timeStep, stafolders);
	}
	else
	{
		LogMessage::log("Cell type not yet supported.");
		std::cout << "Cell type not yet supported." << std::endl;
	}
}